package controllers

import (
	"net/http"
	"spaceofcheng/services/service"
)

// Authenticate 权限验证
// 其中 token 的处理方式是
// 1、获取 request 中 token
// 2、放入 Context 中
// 3、校验 token
// 4、结束后，设置过期并从 Context 中删除
// 5、生成新的 token， 并放入 Context 中
func (c *BaseController) authenticate() {
	c.serv = service.NewSysServ(c.Context)

	// 鉴权 - 并初始化上下文
	res := c.serv.AuthAndInitCtx(c.Ctx)

	if res != nil {
		code := http.StatusOK
		if res["code"] != nil {
			code = res["code"].(int)
		}

		if code != http.StatusOK {
			err := res["error"].(error)
			data := map[string]interface{}{}

			if res["message"] != nil {
				data = res["message"].(map[string]interface{})
			}

			c.ResponseData(data, err, code)
		}
	}
}

// SetTokenExipre 设置 token 过期
func (c *BaseController) SetTokenExipre(uid string) {
	token := c.Context.Get("token")
	if token != nil {
		c.serv.UpdateTokenExpire(token.(string), uid)
	}

	c.Context.Set("token", "")
}

// CreateNewToken 新 token
func (c *BaseController) CreateNewToken() {

	batchNo := ""
	if c.Context.Get("token-batch") != nil {
		batchNo = c.Context.Get("token-batch").(string)
	}

	c.Context.Set("token", c.serv.NewToken(batchNo))
}
