package cases

import (
	"errors"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/cases"
	"spaceofcheng/services/utils"
)

// CaseController 信息
type CaseController struct {
	dao *cases.CaseServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *CaseController) Constructor() {
	c.dao = cases.NewCaseServ(c.Context)
}

// GetCaseList 获取案场列表
func (c *CaseController) GetCaseList() {
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseIDs := c.GetCaseIDs(cases)
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	casename := c.GetString("name")

	list, err := c.dao.GetCases(caseIDs, casename, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetCaseByID 根据ID获取案场信息
func (c *CaseController) GetCaseByID() {
	caseid := c.GetString(":id")
	info, err := c.dao.GetCaseByID(caseid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(info)
}

// SaveCase 保存案场信息
func (c *CaseController) SaveCase() {
	info := model.SysCase{}
	if err := c.ParseForm(&info); err != nil {
		c.ResponseError(err)
	}
	newinfo, err := c.dao.SaveCase(info)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newinfo)
}

//DelCase 删除案场信息
func (c *CaseController) DelCase() {
	caseid := c.GetString(":id")
	err := c.dao.DelCase(caseid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("删除成功！")
}

// GetOrgCases 获取机构案场
func (c *CaseController) GetOrgCases() {
	org := c.Context.Get("org").(model.SysOrg)

	res, err := c.dao.GetAllCasesAndSales(org.OrgId)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(res)
}

// GetCaseConf 获取案场对应的配置
func (c *CaseController) GetCaseConf() {
	caseid := c.GetString(":caseid")
	conf, err := c.dao.GetCaseConf(caseid, cases.CONFTYPE_GOODS)
	if err != nil {
		utils.LogError("获取案场配置信息失败: " + err.Error())
		c.ResponseError(errors.New("获取案场信息失败！"))
	}
	c.ResponseJSON(conf)
}

// OpenCaseCoffee 开启
func (c *CaseController) OpenCaseCoffee() {
	caseid := c.GetString(":caseid")
	err := c.dao.UpdateCaseConf(caseid, cases.CONFTYPE_GOODS, models.STATUS_NORMAL)
	if err != nil {
		utils.LogError("开启失败: " + err.Error())
		c.ResponseError(errors.New("操作失败！"))
	}
	c.ResponseJSON("操作成功！")
}

// CloseCaseCoffee 关闭
func (c *CaseController) CloseCaseCoffee() {
	caseid := c.GetString(":caseid")
	err := c.dao.UpdateCaseConf(caseid, cases.CONFTYPE_GOODS, models.STATUS_READY)
	if err != nil {
		utils.LogError("关闭失败: " + err.Error())
		c.ResponseError(errors.New("操作失败！"))
	}
	c.ResponseJSON("操作成功！")
}
