package cases

import (
	"spaceofcheng/services/models/model"
)

// GetCaseUserByCase 根据案场获取用户信息
func (c *CaseController) GetCaseUserByCase() {
	caseid := c.GetString("caseid")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")

	list, err := c.dao.GetCaseUserByCase(caseid, page, pageSize)

	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// GetCaseUserByType 根据类型获取用户信息
func (c *CaseController) GetCaseUserByType() {
	usertype := c.GetString("usertype")
	caseid := c.GetString("caseid")
	caseuser, err := c.dao.GetCaseUserByType(caseid, usertype)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(caseuser)
}

// GetCaseUserByID 根据ID获取用户
func (c *CaseController) GetCaseUserByID() {
	userid := c.GetString(":userid")
	info, err := c.dao.GetCaseUserByID(userid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(info)
}

// SaveCaseUser 保存案场人员
func (c *CaseController) SaveCaseUser() {
	user := model.SysCaseUser{}
	if err := c.ParseForm(&user); err != nil {
		c.ResponseError(err)
	}
	newUser, err := c.dao.SaveCaseUser(user)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newUser)
}

// DelCaseUser 删除案场人员
func (c *CaseController) DelCaseUser() {
	tagid := c.GetString(":userid")
	err := c.dao.DelCaseUser(tagid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("删除成功！")
}

// GetCaseUserType 获取案场人员类型
func (c *CaseController) GetCaseUserType() {
	orgid := c.GetString("orgid")
	if orgid == "" {
		org := c.Context.Get("org").(model.SysOrg)
		orgid = org.OrgId
	}
	types, err := c.dao.GetUserType(orgid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(types)
}
