package cases

import (
	"errors"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/cases"
	"spaceofcheng/services/utils"
)

// CaseController 信息
type SigninController struct {
	dao *cases.SigninServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *SigninController) Constructor() {
	c.dao = cases.NewSigninServ(c.Context)
}

/**
 条件:
				 selectType: 今天,本周,本月,今年,全部
				 caseid: 案场ID
*/
func (c *SigninController) GetSigninWhere() {
	selectType, err := c.GetInt("selectType")
	page, err := c.GetInt("page")
	pageSize, err := c.GetInt("pageSize")

	caseids := c.GetString("caseid")
	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	res, err := c.dao.GetSigninWhere(selectType, caseids, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(res)
}

// AddSignin 存入签到记录
func (c *SigninController) AddSignin() {
	phone := c.GetString("tel")
	caseId := c.GetString("fieldId")

	signin := model.TaCheckinRecord{}
	signin.Phone = phone

	// 机构
	org := c.Context.Get("org").(model.SysOrg)
	signin.OrgId = org.OrgId

	res, err := c.dao.AddSignin(signin, caseId, "")

	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(res)
}

// GetSigninExcel 导出签到记录表
func (c *SigninController) GetSigninExcel() {

	selectType, err := c.GetInt("selectType")

	// 案场
	caseids := c.GetString("caseid")
	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}

	// 查询出数据
	list, err := c.dao.GetSigninExcel(selectType, caseids)
	if err != nil {
		c.ResponseError(err)
	}

	excel, err := utils.NewTinyXLSXEngine()
	if err != nil {
		utils.LogError("初始化Excel服务失败: " + err.Error())
		c.ResponseError(errors.New("初始化Excel服务失败"))
	}

	// 创建表头
	excel.SetCell(excel.InsertRow(), []string{
		"案场",
		"签到时间",
		"微信昵称",
		"客户姓名",
		"客户类型",
		"手机号",
	})

	// 遍历数据
	for _, item := range list {
		row := excel.InsertRow()

		userTypeName := ""
		if item.UserId == "" {
			userTypeName = "前台用户"
		} else {
			userTypeName = "后台用户"
		}

		dateToString := item.CreateDate.Format("2006-01-02 15:04:05")

		excel.SetCell(row, []string{
			item.CaseName,
			dateToString,
			item.CustomerName,
			item.Name,
			userTypeName,
			item.Phone,
		})
	}
	c.SaveToExcel("签到记录列表.xlsx", excel)

}
