package customerremark

import (
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/customerremark"
)

// CaseController 信息
type CustomerRemarkController struct {
	dao *customerremark.CustomerRemarkServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *CustomerRemarkController) Constructor() {
	c.dao = customerremark.NewCustomerRemarkServ(c.Context)
}

func (c *CustomerRemarkController) GetCustomerRemarkList() {
	salesId := c.GetString(":salesId")
	customerId := c.GetString(":customerId")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	list, err := c.dao.GetCustomerRemarkList(salesId, customerId, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

func (c *CustomerRemarkController) GetCustomerReceiveRecord() {
	salesId := c.GetString(":salesId")
	customerId := c.GetString(":customerId")
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	list, err := c.dao.GetCustomerReceiveRecord(salesId, customerId, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

func (c *CustomerRemarkController) AddRemark() {
	remark := model.TaSaleCustomerRemark{}
	if err := c.ParseForm(&remark); err != nil {
		c.ResponseError(err)
	}
	newRemark, err := c.dao.AddRemark(remark)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newRemark)
}

func (c *CustomerRemarkController) IsExist() {
	recommendCode := c.GetString(":recommendCode")
	num, err := c.dao.IsExist(recommendCode)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(num)
}

func (c *CustomerRemarkController) SearchCustomer() {
	salesId := c.GetString(":salesId")
	customerInfo := c.GetString(":customerInfo")
	customer, err := c.dao.SearchCustomer(customerInfo, salesId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(customer)
}
