package controllers

import (
	"bytes"
	"errors"
	"spaceofcheng/services/utils"
	"strconv"
	"time"

	"net/http"
	"net/url"
)

// FileUpload 文件上传
func (c *BaseController) FileUpload() {
	file, err := c.uploadFileToOSS("file")
	if err != nil {
		c.ResponseError(err, http.StatusInternalServerError)
	}

	c.ResponseJSON(map[string]interface{}{
		"url": file,
	})
}

func (c *BaseController) uploadFileToOSS(field string) (string, error) {
	aliConf, ok := c.Configer[AliYunConf]
	if !ok {
		return "", errors.New("没有找到阿里云相关配置")
	}

	endpoint := aliConf.String("oss::Endpoint")
	accessKeyID := aliConf.String("oss::AccessKeyId")
	accessKeySecret := aliConf.String("oss::AccessKeySecret")
	bucket := aliConf.String("oss::Bucket")

	aliCli, err := utils.GetOssClient(endpoint, accessKeyID, accessKeySecret)
	if err != nil {
		return "", err
	}

	fNameExtra := strconv.FormatInt(time.Now().Unix(), 10)

	fileURL, err := utils.UploadFileToBucket(aliCli, c.Ctx.Request, bucket, field, fNameExtra)
	if err != nil {
		return "", err
	}

	return fileURL, nil
}

// SaveToExcel 保存文件到 excel
// 文件不落地, 对内存开销比较大, 如果文件过大, 服务可能会崩掉
func (c *BaseController) SaveToExcel(fn string, excel *utils.TinyXLSXEngine) {
	var buf bytes.Buffer
	if err := excel.Write(&buf); err != nil {
		utils.LogError("写 xlsx buffer 失败: " + err.Error())
		c.ResponseError(errors.New("生成 excel 异常, 请重试"))
	}

	c.Ctx.Output.Header("Content-Disposition", "attachment; filename="+url.QueryEscape(fn))
	c.Ctx.Output.Header("Content-Description", "File Transfer")
	c.Ctx.Output.ContentType(".xlsx")
	c.Ctx.Output.Header("Content-Transfer-Encoding", "binary")
	c.Ctx.Output.Header("Expires", "0")
	c.Ctx.Output.Header("Cache-Control", "must-revalidate")
	c.Ctx.Output.Header("Pragma", "public")

	r := bytes.NewReader(buf.Bytes())
	http.ServeContent(c.Ctx.ResponseWriter, c.Ctx.Request, fn, time.Now().Local(), r)

	c.destroyContext(true)
	c.StopRun()
}
