package luckdrawlist

import (
	"errors"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/luckdrawlist"
)

// LuckdrawlistController 信息
type LuckdrawlistController struct {
	dao *luckdrawlist.LuckdrawlistServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *LuckdrawlistController) Constructor() {
	c.dao = luckdrawlist.NewLuckdrawlistServ(c.Context)
}
func (c *LuckdrawlistController) GetLuckdrawList() {
	caseid := c.GetString("caseid")
	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")
	if page < 0 {
		page = 1
	}

	if pagesize <= 0 {
		pagesize = 10
	}
	list, total, err := c.dao.GetLuckdrawList(caseid, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"list":     list,
		"page":     page,
		"pagesize": pagesize,
		"pagenum":  total,
	})
}

func (c *LuckdrawlistController) GetLuckShareList() {
	caseid := c.GetString("caseid")
	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")
	if page < 0 {
		page = 1
	}

	if pagesize <= 0 {
		pagesize = 10
	}
	toPhone := c.GetString("toPhone")
	fromPhone := c.GetString("fromPhone")
	list, total, err := c.dao.GetLuckShareList(caseid, fromPhone, toPhone, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"list":     list,
		"page":     page,
		"pagesize": pagesize,
		"pagenum":  total,
	})

}

func (c *LuckdrawlistController) GetVerifyList() {
	luckdrawId := c.GetString(":luckdrawId")
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseids := c.GetCaseIDs(cases)
	if luckdrawId == "" {
		c.ResponseError(errors.New("无效二维码"))
	}
	luckdraw, err := c.dao.GetVerifyList(luckdrawId, caseids)
	if err != nil {
		c.ResponseError(err)
	}
	if luckdraw == nil {
		c.ResponseError(errors.New("您没有可以核销的奖品"))
	}
	c.ResponseJSON(luckdraw)
}

func (c *LuckdrawlistController) VerifyLuckdraw() {
	sysUser := c.Context.Get("user").(model.SysUser)
	userId := sysUser.UserId
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseids := c.GetCaseIDs(cases)
	luckdrawId := c.GetString(":luckdrawId")
	if luckdrawId == "" {
		c.ResponseError(errors.New("无效二维码"))
	}
	err := c.dao.VerifyLuckdraw(luckdrawId, caseids, userId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("核销成功")

}
