package marketing

import (
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/marketing"
)

// CourseController 商品
type MarketingController struct {
	serv *marketing.MarketingServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *MarketingController) Constructor() {
	c.serv = marketing.NewMarketingServ(c.Context)
}

// 获取 营销活动列表
func (c *MarketingController) GetMarketingList() {
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pageSize")
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseIDs := c.GetCaseIDs(cases)
	marketings, err := c.serv.GetMarketingList(caseIDs, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(marketings)

}

// SaveMarketing 保存营销活动
func (c *MarketingController) SaveMarketing() {
	marketing := model.SysActivity{}
	// 机构
	//org := c.Context.Get("org").([]model.SysOrg)
	// 获取值
	activityName := c.GetString("activityName")
	activityType := c.GetString("activityType")
	activeType := c.GetString("activeType")
	resourceDesc := c.GetString("resourceDesc") // 活动描述
	caseId := c.GetString("CaseId")

	// 赋值
	marketing.ActivityName = activityName
	marketing.ActivityType = activityType
	marketing.CaseId = caseId

	newInfo, _, err := c.serv.SaveMarketing(marketing, activeType, resourceDesc)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(newInfo)
}

// GetMarketingById 根据activityId查询
func (c *MarketingController) GetMarketingById() {
	activityId := c.GetString("activityId")

	info, err := c.serv.GetMarketingById(activityId)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(info)
}

// UpdateMarketing 修改数据
func (c *MarketingController) UpdateMarketing() {
	activity := model.SysActivity{}

	// 获取值
	activityName := c.GetString("activityName")
	activityType := c.GetString("activityType")

	// 赋值
	activity.ActivityName = activityName
	activity.ActivityType = activityType

	err := c.serv.UpdateMarketing(activity)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功!")
}

// DelMarketing 删除
func (c *MarketingController) DelMarketing() {
	activityId := c.GetString(":activityId")
	err := c.serv.DelMarketing(activityId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功!")
}

// SetMarketingNormal 正常
func (c *MarketingController) SetMarketingNormal() {
	activityId := c.GetString(":activityId")
	err := c.serv.SetMarketingNormal(activityId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功!")
}

// SetMarketingDisable 停用
func (c *MarketingController) SetMarketingDisable() {
	activityId := c.GetString(":activityId")
	err := c.serv.SetMarketingDisable(activityId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功!")
}

func (c *MarketingController) GetUserCases() {
	cases := c.Context.Get("cases").([]model.SysUserCase)
	caseIDs := c.GetCaseIDs(cases)
	sysCase, err := c.serv.GetUserCases(caseIDs)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(sysCase)
}
