package bodycheck

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"

	"github.com/go-xorm/xorm"
)

// DAO 当前数据库操作对象
type DAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewDAO 初始化DAO
func NewDAO(ctx *utils.Context) *DAO {
	return &DAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// GetBodyCheckByUser 根据用户信息获取体检信息
func (m *DAO) GetBodyCheckByUser(userID string) ([]model.TaBodyCheck, error) {
	var bodychecks []model.TaBodyCheck
	var err error
	err = m.db.Where("user_id=?", userID).And("status=?", models.STATUS_NORMAL).Desc("create_date").Find(&bodychecks)
	return bodychecks, err
}

// GetCheckSpecs 获取项目指标信息
func (m *DAO) GetCheckSpecs() ([]model.TdCheckSpec, error) {
	var specs []model.TdCheckSpec
	err := m.db.Find(&specs)
	return specs, err
}

// GetCheckByUserAndEquipmentID 根据用户与设备ID获取体检信息
func (m *DAO) GetCheckByUserAndEquipmentID(userID string, EquipmentID string) (*model.TaBodyCheck, error) {
	var check []model.TaBodyCheck
	err := m.db.Where("user_id=?", userID).And("equipment_id=?", EquipmentID).And("status=?", models.STATUS_NORMAL).Desc("create_date").Find(&check)
	if err != nil {
		return nil, err
	}
	if len(check) > 0 {
		return &check[0], nil
	}
	return nil, nil
}

// SaveBodyCheckInfo 用户体检信息新增
func (m *DAO) SaveBodyCheckInfo(checkNew model.TaBodyCheck) (*model.TaBodyCheck, error) {
	checkNew.Status = models.STATUS_NORMAL
	checkNew.Id = utils.GetGUID()
	// checkNew.CreateDate = time.Now()
	_, err := m.db.Insert(&checkNew)
	return &checkNew, err
}

// GetCaseEquipment 获取设备案场信息
func (m *DAO) GetCaseEquipment(EquipmentID string) (*model.SysCaseEquipment, error) {
	var caseEquipment model.SysCaseEquipment

	_, err := m.db.Where("equipment_code=?", EquipmentID).And("status=?", models.STATUS_NORMAL).Get(&caseEquipment)
	return &caseEquipment, err
}

type Presentation struct {
	model.TaPresentation `xorm:"extends"`
	TypeName             string
	Details              []model.TaPresentationDetail
}

// GetPresentationByCheckID 根据体检ID获取项目信息
func (m *DAO) GetPresentationByCheckID(checkID string) ([]Presentation, error) {
	var presentations []Presentation
	sql := `select a.*,b.type_name from ta_presentation a inner join td_check_type b on a.check_type=b.id where a.check_id='` + checkID + `'`

	err := m.db.Sql(sql).Find(&presentations)
	if err != nil {
		return nil, err
	}
	var Info []Presentation
	for _, presentation := range presentations {
		details, err := m.GetDetailsByPresentID(presentation.Id)
		if err != nil {
			return nil, err
		}
		presentation.Details = details
		Info = append(Info, presentation)
	}

	return Info, nil
}

// GetDetailsByPresentID 根据项目ID获取详情
func (m *DAO) GetDetailsByPresentID(ID string) ([]model.TaPresentationDetail, error) {
	var details []model.TaPresentationDetail
	err := m.db.Where("presentation_id=?", ID).Find(&details)
	return details, err
}

// GetPresentation 获取体检项目信息
func (m *DAO) GetPresentation(checkID string, typeID int) (*model.TaPresentation, error) {
	var presentation model.TaPresentation
	_, err := m.db.Where("check_id=?", checkID).And("check_type='" + strconv.Itoa(typeID) + "'").Get(&presentation)
	return &presentation, err
}

// SavePresentation 新增体检项目信息
func (m *DAO) SavePresentation(info model.TaPresentation) (*model.TaPresentation, error) {
	info.Id = utils.GetGUID()
	_, err := m.db.Insert(&info)
	return &info, err
}

// UpdatePresentation 更新体检项目信息
func (m *DAO) UpdatePresentation(info *model.TaPresentation) error {
	cols := []string{
		"check_date",
		"check_result",
		"report_url",
	}
	_, err := m.db.Cols(cols...).Where("id = ?", info.Id).Update(info)
	return err
}

// SavePresentationDetail 保存体检项目明细
func (m *DAO) SavePresentationDetail(details []model.TaPresentationDetail) error {
	_, err := m.db.Insert(&details)
	return err
}

// DeletePresentionDetail 根据项目ID删除明细信息
func (m *DAO) DeletePresentionDetail(ID string) error {
	sql := "delete from ta_presentation_detail where presentation_id=?"
	_, err := m.db.Exec(sql, ID)
	return err
}
