package customer

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/yl10/kit/guid"

	"github.com/go-xorm/xorm"
)

// CustomerDAO 当前数据库操作对象
type CustomerDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewCustomerDAO New Inst
func NewCustomerDAO(ctx *utils.Context) *CustomerDAO {
	return &CustomerDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// CustWithWXList 客户列表
func (m *CustomerDAO) CustWithWXList(phone, username, recommendname, begindate, enddate, caseids string, page ...int) ([]CustWithWX, int64, error) {
	org := m.ctx.Get("org").(model.SysOrg)

	pageNum := 10
	if m.ctx.Get("pageNum") != nil {
		pageNum = m.ctx.Get("pageNum").(int)
	}
	if len(page) > 1 {
		pageNum = page[1]
	}

	pg := 1
	if len(page) > 0 {
		pg = page[0]
	}
	offset := (pg - 1) * pageNum
	if offset < 0 {
		offset = 0
	}

	query := `
		SELECT
			t.*, s.account_info,
			s.account_type,
			s.mapping_id,
			s.openid,
			s.user_id as map_user,
			s.uuid
		FROM
			ta_customer t
		LEFT JOIN ta_user_mapping s ON t.customer_id = s.user_id
			AND s.account_type = '` + models.ACCMAP_WECHAT + `'
		WHERE t.status > ? AND t.org_id = '` + org.OrgId + `'
		AND (t.recommend_case in ('` + strings.Replace(caseids, ",", "','", -1) + `') or t.recommend_case = '' or t.recommend_case is null)
		`
	if phone != "" {
		query += " and t.phone like '%" + phone + "%'"
	}

	if username != "" {
		query += " and t.customer_name like '%" + username + "%'"
	}

	if recommendname != "" {
		query += " AND t.recommend_name like '%" + recommendname + "%'"
	}

	if begindate != "" {
		query = query + ` AND t.create_date>'` + begindate + `'`
	}

	if enddate != "" {
		query = query + ` AND t.create_date<'` + begindate + `'`
	}

	query = query + ` ORDER BY
			t.create_date DESC
	`
	var cust []CustWithWX

	cnt, err := utils.NewPageNaviEngine(m.ctx).GetPageList(&cust, query, []int{pageNum, offset}, models.STATUS_DEL)
	if err != nil {
		return nil, 0, err
	}

	return cust, cnt, nil
}

// GetCustWithWXByID 获取客户(微信)信息
func (m *CustomerDAO) GetCustWithWXByID(custID string) (*CustWithWX, error) {
	if custID == "" {
		return nil, errors.New("获取客户信息失败")
	}

	cust := CustWithWX{}
	query := `
		SELECT *, t.user_id as map_user
		FROM
			ta_customer t
		LEFT JOIN ta_user_mapping s ON t.customer_id = s.user_id
			AND s.account_type = '` + models.ACCMAP_WECHAT + `'
		WHERE
			t.customer_id = ?
	`

	if _, err := m.db.SQL(query, custID).Get(&cust); err != nil {
		return nil, err
	}

	return &cust, nil
}

// GetCustWithWXByOpenID 根据openid获取用户信息
func (m *CustomerDAO) GetCustWithWXByOpenID(openid string) (*CustWithWX, error) {
	if openid == "" {
		return nil, errors.New("获取客户信息失败")
	}

	cust := CustWithWX{}
	query := `
	SELECT a.*,b.*, b.user_id as map_user,c.type_id as user_type_id,e.real_name,e.recommend_code
	FROM
		ta_user_mapping a
	LEFT JOIN ta_customer b ON a.user_id = b.customer_id
	LEFT JOIN sys_user e ON b.user_id = e.user_id
	AND a.account_type = '` + models.ACCMAP_WECHAT + `'
	LEFT JOIN sys_user_type c on b.user_id = c.user_id
		WHERE
			a.openid = ?
	`

	if _, err := m.db.SQL(query, openid).Get(&cust); err != nil {
		return nil, err
	}

	return &cust, nil
}

// GetCustomerByPhone 依据手机号获取客户信息
func (m *CustomerDAO) GetCustomerByPhone(phone string) (*model.TaCustomer, error) {
	custs := []model.TaCustomer{}

	if err := m.db.Where("phone=?", phone).And("status>?", models.STATUS_DEL).Find(&custs); err != nil {
		return nil, err
	}

	if len(custs) == 0 {
		return nil, nil
	}

	if len(custs) > 1 {
		return nil, errors.New("数据异常, 一个手机绑定了多个用户")
	}

	cust := custs[0]

	return &cust, nil
}

// UpdateUserMapping 更新用户映射
func (m *CustomerDAO) UpdateUserMapping(userMap *model.TaUserMapping, cols []string) error {
	if userMap.MappingId == "" {
		return errors.New("用户映射信息不存在")
	}

	_, err := m.db.Where("mapping_id=?", userMap.MappingId).Cols(cols...).Update(userMap)
	return err
}

// UpdateCustomer 更新用户信息
func (m *CustomerDAO) UpdateCustomer(customer *model.TaCustomer, cols []string) error {
	if customer.CustomerId == "" {
		return errors.New("用户信息不存在")
	}
	_, err := m.db.Where("customer_id=?", customer.CustomerId).Cols(cols...).Update(customer)
	return err
}

// GetCaseByID 获取案场
// func (m *CustomerDAO) GetCaseByID(caseID string) (*model.SysCase, error) {
// 	if caseID == "" {
// 		return nil, errors.New("获取案场信息失败")
// 	}

// 	cs := model.SysCase{}
// 	has, err := m.db.Where("case_id=?", caseID).Get(&cs)
// 	if err != nil {
// 		return nil, err
// 	}

// 	if !has {
// 		return nil, nil
// 	}

// 	return &cs, nil
// }

// GetUserByID 获取系统用户
func (m *CustomerDAO) GetUserByID(userID string) (*model.SysUser, error) {
	if userID == "" {
		return nil, errors.New("获取相关人员信息失败")
	}

	user := model.SysUser{}
	has, err := m.db.Where("user_id=?", userID).Get(&user)
	if err != nil {
		return nil, err
	}

	if !has {
		return nil, nil
	}

	return &user, nil
}

// GetUserByPhone 依据手机号获取人员信息
func (m *CustomerDAO) GetUserByPhone(phone string) (*model.SysUser, error) {
	user := model.SysUser{}

	sql := `select a.* from sys_user a inner join sys_user_type b on a.user_id=b.user_id
	where a.phone=? and a.status>? and b.type_id in ('` + models.USERTYPE_MANAGER + `','` + models.USERTYPE_SALES + `')`
	has, err := m.db.Sql(sql, phone, models.STATUS_DEL).Get(&user)
	if err != nil {
		return nil, err
	}

	if !has {
		return nil, nil
	}

	return &user, nil
}

// SaveCustomer 新增客户
func (m *CustomerDAO) SaveCustomer(cust *model.TaCustomer) error {
	cust.CustomerId = guid.NewGUIDString()
	cust.CreateDate = time.Now().Local()
	cust.Status = models.STATUS_NORMAL

	_, err := m.db.Insert(cust)
	return err
}

// GetCustomerByID 依据ID获取客户
func (m *CustomerDAO) GetCustomerByID(id string) (*model.TaCustomer, error) {
	cust := new(model.TaCustomer)

	if _, err := m.db.Where("customer_id=?", id).Get(cust); err != nil {
		return nil, err
	}

	return cust, nil
}

func (m *CustomerDAO) GetSalesByRecommendCode(recommendCode string) (*model.SysUser, error) {
	if recommendCode == "" {
		return nil, errors.New("获取相关人员信息失败")
	}
	user := model.SysUser{}
	has, err := m.db.Where("recommend_code=?", recommendCode).Get(&user)
	if err != nil {
		return nil, err
	}

	if !has {
		return nil, nil
	}

	return &user, nil
}

func (m *CustomerDAO) GetSalesCaseById(userId string) (*model.SysUserCase, error) {
	var userCase []model.SysUserCase
	sql := `SELECT
	* 
FROM
	sys_user_case 
WHERE
	is_belong = 1 
	AND user_id = '` + userId + `'and status > ` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&userCase)
	if len(userCase) <= 0 {
		return nil, err
	}
	return &userCase[0], err
}
