package goods

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/yl10/kit/guid"
)

const (
	USERTPE_SALES     = "sales"
	USERTYPE_CUSTOMER = "customer"
	USERTYPE_MANAGER  = "manager"

	INTIMIDATE_NO  = 0
	INTIMIDATE_YES = 1
)

// GetOrdersByCases 获取案场订单列表
func (m *GoodsDAO) GetOrdersByCases() error {
	return nil
}

// SaveOrders 下单
func (m *GoodsDAO) SaveOrders(order *model.TaGoodsOrders) error {
	if order.UserId == "" {
		return errors.New("没有下单人信息")
	}

	caseID := order.CaseId
	if caseID == "" {
		return errors.New("没有指定案场")
	}

	caseInfo, err := m.getCaseByID(caseID)
	if err != nil {
		return err
	}

	now := time.Now().Local()

	// order.OrdersId = guid.NewGUIDString()
	order.OrgId = caseInfo.OrgId
	order.CreateDate = now
	order.CouponAmount = "0"
	order.IsIntimidate = INTIMIDATE_NO
	order.DupKey = utils.GetFiveSeconds(now) + order.UserId + order.TableId + strconv.Itoa(order.Status)

	// 当前订单号的随机方式 = 时间 + 个人ID
	ordersno := "G-" + now.Format("20060102150405") + "-" + strings.Join(utils.GUIID2IntString(order.UserId), "")
	if len(ordersno) > 32 {
		ordersno = string([]byte(ordersno)[:32])
	}
	order.OrdersNo = ordersno
	if _, err := m.db.Insert(order); err != nil {
		utils.LogError("保存订单主记录失败: " + err.Error())

		return errors.New("保存订单主记录失败")
	}
	return nil
}

// UpdateOrdersIntimidate 更新订单已打单
func (m *GoodsDAO) UpdateOrdersIntimidate(id string) error {
	var orders = model.TaGoodsOrders{
		OrdersId:     id,
		IsIntimidate: INTIMIDATE_YES,
	}
	var cols = []string{
		"orders_id",
		"is_intimidate",
	}

	return m.UpdateOrders(&orders, cols)
}

// UpdateOrders  更新订单
func (m *GoodsDAO) UpdateOrders(orders *model.TaGoodsOrders, cols []string) error {
	_, err := m.db.Cols(cols...).Where("orders_id=?", orders.OrdersId).Update(orders)
	return err
}

// SaveOrdersDetail 保存订单明细
func (m *GoodsDAO) SaveOrdersDetail(list []model.TaGoodsOrdersDetail, ordersID string) error {
	for inx := range list {
		list[inx].DetailId = guid.NewGUIDString()
		list[inx].OrdersId = ordersID
		list[inx].CreateDate = time.Now()
	}

	if _, err := m.db.Insert(list); err != nil {
		utils.LogError("保存订单明细失败: " + err.Error())

		return errors.New("保存订单明细失败")
	}

	return nil
}

// SaveOrdersCoupon 保存订单优惠券
func (m *GoodsDAO) SaveOrdersCoupon(coupon *model.TaGoodsOrdersCoupon, order *model.TaGoodsOrders) error {
	if order.OrdersId == "" {
		return errors.New("保存订单券失败, 订单不存在")
	}
	coupon.OrdersCouponId = utils.GetGUID()
	coupon.Status = models.STATUS_NORMAL
	coupon.CreateDate = time.Now().Local()
	_, err := m.db.Insert(coupon)

	return err
}

// GetCouponByOrdersID 获取优惠券订单
func (m *GoodsDAO) GetCouponByOrdersID(ordersID string) ([]model.TaCoupon, error) {
	var res []model.TaCoupon
	query := `
		SELECT t.*
		FROM ta_coupon t
		JOIN ta_goods_orders_coupon s ON t.coupon_id = s.coupon_id
		WHERE s.orders_id = ?
		ORDER BY t.create_date
	`

	err := m.db.SQL(query, ordersID).Find(&res)
	return res, err
}

func (m *GoodsDAO) getCaseByID(caseID string) (*model.SysCase, error) {
	cs := new(model.SysCase)
	if _, err := m.db.Where("case_id=?", caseID).Get(cs); err != nil {
		return nil, err
	}

	return cs, nil
}

// GetOrdersByCaseID 获取订单列表
func (m *GoodsDAO) GetOrdersByCaseID(caseID, userType string, makeStatus []string, orderDate []time.Time, limit []int) ([]OrdersWithGoods, int64, error) {
	var orderList []OrdersWithGoods

	query := "select a.*,b.area_icon,b.area_icon_white from ta_goods_orders a inner join sys_case_area b on a.area_id=b.area_id where a.case_id = ? and a.status = ?"

	// 制作状态
	if makeStatus != nil && len(makeStatus) > 0 {
		query += ` and a.make_status in ('` + strings.Join(makeStatus, "','") + `')`
	}

	// 用户类型
	if userType != "" {
		query += ` and a.user_type = '` + userType + `'`
	}

	// 订单起始时间
	if orderDate != nil && len(orderDate) > 0 {
		query += ` and DATE_FORMAT(a.create_date, '%Y-%m-%d %H:%i') >= '` + orderDate[0].Format("2006-01-02 15:04") + `'`

		if len(orderDate) > 1 {
			query += ` and DATE_FORMAT(a.create_date, '%Y-%m-%d %H:%i') <= '` + orderDate[1].Format("2006-01-02 15:04") + `'`
		}
	}

	query += " order by a.create_date asc"

	var cnt int64
	if limit != nil && len(limit) > 1 {
		var err error
		cnt, err = utils.NewPageNaviEngine(m.ctx).GetPageList(&orderList, query, limit, caseID, models.STATUS_NORMAL)
		if err != nil {
			return nil, 0, err
		}
	} else {
		if err := m.db.SQL(query, caseID, models.STATUS_NORMAL).Find(&orderList); err != nil {
			return nil, 0, err
		}
	}

	for i, o := range orderList {
		details, err := m.GetOrderDetailByOrder(o.OrdersId)
		if err != nil {
			return nil, 0, err
		}
		orderList[i].Goods = details
	}

	return orderList, cnt, nil
}

// GetOrderDetailByOrder 获取商品明细
func (m *GoodsDAO) GetOrderDetailByOrder(orderid string) ([]DetailWithType, error) {
	var details []DetailWithType
	sql := `select a.*,b.type_id,c.type_name from ta_goods_orders_detail a inner join ta_goods b on a.goods_id=b.goods_id inner join td_goods_type c on b.type_id=c.type_id where a.orders_id=?`
	if err := m.db.Sql(sql, orderid).Find(&details); err != nil {
		return nil, err
	}
	return details, nil
}

func (m *GoodsDAO) GetOrdersByID(id string) (*OrdersWithGoods, error) {
	query := `select a.*,b.area_icon,b.area_icon_white from ta_goods_orders a inner join sys_case_area b on a.area_id=b.area_id where a.orders_id = ?`

	res := new(OrdersWithGoods)
	if _, err := m.db.SQL(query, id).Get(res); err != nil {
		return nil, err
	}

	details, err := m.GetOrderDetailByOrder(res.OrdersId)
	if err != nil {
		return nil, err
	}

	res.Goods = details
	return res, nil
}

func (m *GoodsDAO) GetOrdersDetailByID(id string, status ...int) (*OrdersDetail, error) {
	sta := models.STATUS_NORMAL
	if len(status) > 0 {
		sta = status[0]
	}

	res := new(OrdersDetail)
	if _, err := m.db.Table("ta_goods_orders").
		Where("orders_id=?", id).
		And("status=?", sta).
		Get(res); err != nil {
		return nil, err
	}

	details, err := m.GetOrderDetailByOrder(res.OrdersId)
	if err != nil {
		return nil, err
	}

	res.Goods = details
	return res, nil
}

// GetOrdersByRecord 根据
func (m *GoodsDAO) GetOrdersByRecord(recordid string) ([]OrdersWithGoods, error) {
	var orderList []OrdersWithGoods
	sql := `select a.*, c.case_name from ta_goods_orders a inner join sys_case_record b on a.case_id=b.case_id and date_format(a.create_date,'%Y-%m')=b.month
	inner join sys_case c on a.case_id=c.case_id
	where b.record_id='` + recordid +
		`' and a.user_type != '` + USERTYPE_CUSTOMER + `' and b.status=` + strconv.Itoa(models.STATUS_NORMAL) +
		` and a.status=` + strconv.Itoa(models.STATUS_NORMAL) + ` and a.pay_type = '` + models.CONSUME_INNER + `' order by a.create_date desc`
	err := m.db.Sql(sql).Find(&orderList)
	if err != nil {
		return nil, err
	}
	for i, o := range orderList {
		details, err := m.GetOrderDetailByOrder(o.OrdersId)
		if err != nil {
			return nil, err
		}
		orderList[i].Goods = details
	}
	return orderList, nil
}

// GetCustomerOrders 获取用户订单
func (m *GoodsDAO) GetCustomerOrders(custID string, page, pageSize int, fromClient string) ([]OrdersWithGoods, error) {
	var orderList []OrdersWithGoods
	sql := `select a.*, c.case_name from ta_goods_orders a 
	inner join sys_case c on a.case_id=c.case_id
	where a.user_id='` + custID +
		`' and a.status=` + strconv.Itoa(models.STATUS_NORMAL)

	if fromClient == "c" {
		sql += " and (a.pay_type > '" + models.CONSUME_INNER + "' or a.pay_type < '" + models.CONSUME_INNER + "' )"
	} else {
		sql += " and a.pay_type = '" + models.CONSUME_INNER + "'"
	}

	sql += ` order by a.create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)

	err := m.db.Sql(sql).Find(&orderList)
	if err != nil {
		return nil, err
	}
	for i, o := range orderList {
		details, err := m.GetOrderDetailByOrder(o.OrdersId)
		if err != nil {
			return nil, err
		}
		orderList[i].Goods = details
	}
	return orderList, nil
}

// GetCustomerOrdersCount 获取用户订单count
func (m *GoodsDAO) GetCustomerOrdersCount(custID, fromClient string) (int, error) {
	var orderList []OrdersWithGoods
	sql := `select a.*, c.case_name from ta_goods_orders a 
	inner join sys_case c on a.case_id=c.case_id
	where a.user_id='` + custID +
		`' and a.status=` + strconv.Itoa(models.STATUS_NORMAL)

	if fromClient == "c" {
		sql += " and (a.pay_type > '" + models.CONSUME_INNER + "' or a.pay_type < '" + models.CONSUME_INNER + "' )"
	} else {
		sql += " and a.pay_type = '" + models.CONSUME_INNER + "'"
	}

	err := m.db.Sql(sql).Find(&orderList)
	if err != nil {
		return 0, err
	}
	return len(orderList), nil
}

// GetOrdersCountByCase 根据案场获取订单信息
func (m *GoodsDAO) GetOrdersCountByCase(caseid, userid string) (int64, error) {
	order := new(model.TaGoodsOrders)
	total, err := m.db.Where("case_id = ?", caseid).And("user_id=?", userid).And("pay_type=?", models.CONSUME_INNER).And("status=?", models.STATUS_NORMAL).SumInt(order, "orders_num")
	return total, err
}

// GetMonthOrdersCountByCase 获取案场本月的订单数量
func (m *GoodsDAO) GetMonthOrdersCountByCase(caseid, userid string) (int64, error) {
	order := new(model.TaGoodsOrders)
	total, err := m.db.Where("case_id = ?", caseid).And("user_id=?", userid).And("pay_type=?", models.CONSUME_INNER).And("status=?", models.STATUS_NORMAL).And("date_format(create_date,'%Y-%m')=date_format(NOW(),'%Y-%m')").SumInt(order, "orders_num")
	return total, err
}

// UpdateOrdersMake 更新订单制作状态
func (m *GoodsDAO) UpdateOrdersMake(orders model.TaGoodsOrders) error {
	cols := []string{
		"make_status",
	}
	_, err := m.db.Cols(cols...).Where("orders_id=?", orders.OrdersId).Update(orders)
	return err
}

// GetOrderDetailByUser 获取用户所有下单信息
func (m *GoodsDAO) GetOrderDetailByUser(caseid, userid string) ([]model.TaGoodsOrdersDetail, error) {
	var details []model.TaGoodsOrdersDetail
	sql := `select a.* from ta_goods_orders_detail a inner join ta_goods_orders b on a.orders_id=b.orders_id
		inner join ta_customer c on b.user_id = c.customer_id
	where c.user_id=? and b.status=? and b.case_id=? and b.pay_type = ? order by b.create_date desc`
	if err := m.db.Sql(sql, userid, models.STATUS_NORMAL, caseid, models.CONSUME_INNER).Find(&details); err != nil {
		return nil, err
	}
	return details, nil
}

// GetOrderNumByUserNow 查询用户一分钟内有无订单
func (m *GoodsDAO) GetOrderNumByUserNow(userId string) (int, error) {
	var order []model.TaGoodsOrders
	sql := `SELECT
	*
FROM
	ta_goods_orders 
WHERE 
	user_id = '` + userId + `' 
	AND DATE_FORMAT( create_date, '%Y-%m-%d %H:%i:%s' ) >= DATE_FORMAT( date_sub( now( ), INTERVAL 5 SECOND ), '%Y-%m-%d %H:%i:%s' ) 
	AND status = ` + strconv.Itoa(models.STATUS_NORMAL)
	err := m.db.Sql(sql).Find(&order)
	return len(order), err
}
