package gymcard

import (
	"fmt"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/xorm"
)

// GymcardDAO 当前数据库操作对象
type GymcardDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewGymcardDAO New Inst
func NewGymcardDAO(ctx *utils.Context) *GymcardDAO {
	return &GymcardDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

type CustomerGym struct {
	model.TaCustomerGym `xorm:"extends"`
	CardImageUrl        string
	CardShareInfo       string
	CardUseInstruction  string
	CardUseRule         string
	Price               string
}

type Gymcard struct {
	model.TaGymCard    `xorm:"extends"`
	CardImageUrl       string
	CardShareInfo      string
	CardUseInstruction string
	CardUseRule        string
	ChannelId          string
}

// GetGymcard 根据案场id获取游泳健身卡列表
func (m *GymcardDAO) GetGymcard(filters []string, limit []int) ([]Gymcard, int64, error) {
	var gymcard []Gymcard
	filterString := ""
	if len(filters) > 0 {
		filterString = strings.Join(filters, " and ")
		filterString += " and "
	}

	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule,
	d.channel_id
FROM
	ta_gym_card a
	INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
	INNER JOIN ta_card_share c ON a.gym_card_id = c.gym_card_id
	INNER JOIN ta_card_coupon_channel d ON a.gym_card_id = d.card_coupon_id
	where %s a.status > ` + strconv.Itoa(models.STATUS_DEL)
	sql += ` order by a.create_date desc `
	total, err := utils.NewPageNaviEngine(m.ctx).GetPageList(&gymcard, fmt.Sprintf(sql, filterString), limit)
	if err != nil {
		return nil, 0, err
	}

	return gymcard, total, nil
}

// GetGymcardCount 根据案场id获取游泳健身卡数量
func (m *GymcardDAO) GetGymcardCount(caseid string) (int, error) {
	var gymcard []Gymcard
	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule
FROM
	ta_gym_card a
	INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
	INNER JOIN ta_card_share c ON a.gym_card_id = c.gym_card_id
	where a.case_id in('` + strings.Replace(caseid, ",", "','", -1) + `')
	and  a.status > ` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&gymcard)
	return len(gymcard), err
}

// GetGymcardById 根据id获取游泳健身卡详情
func (m *GymcardDAO) GetGymcardById(gymcardId string) (*Gymcard, error) {
	var gymcard []Gymcard
	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule,
	d.channel_id
FROM
	ta_gym_card a
	INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
	INNER JOIN ta_card_share c ON a.gym_card_id = c.gym_card_id
	INNER JOIN ta_card_coupon_channel d ON a.gym_card_id = d.card_coupon_id
	where a.gym_card_id = '` + gymcardId + `'
	and  a.status > ` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&gymcard)
	if err != nil {
		return nil, err
	}
	if len(gymcard) > 0 {
		return &gymcard[0], nil
	}
	return nil, nil
}

// AddGymcard 新增游泳健身卡
func (m *GymcardDAO) AddGymcard(gymcard model.TaGymCard) (model.TaGymCard, error) {
	gymcard.GymCardId = utils.GetGUID()
	gymcard.SentCount = 0
	gymcard.UsedCount = 0
	gymcard.Status = models.STATUS_NORMAL
	gymcard.CreateDate = time.Now()
	_, err := m.db.Insert(&gymcard)
	return gymcard, err
}

// AddCardChannel 新增游泳健身卡渠道
func (m *GymcardDAO) AddCardChannel(gymcardId, channelId string) error {
	var cardChannel model.TaCardCouponChannel
	cardChannel.CardCouponChannelId = utils.GetGUID()
	cardChannel.CardCouponId = gymcardId
	cardChannel.CardCouponType = models.GIVE_GIFT_GYM
	cardChannel.ChannelId = channelId
	cardChannel.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(&cardChannel)
	return err

}

// AddGymcardImg 新增游泳健身卡图片
func (m *GymcardDAO) AddGymcardImg(gymcardId, imgUrl string) error {
	var cardImg model.TaCardImage
	cardImg.CardImageId = utils.GetGUID()
	cardImg.CardImageUrl = imgUrl
	cardImg.GymCardId = gymcardId
	cardImg.Sort = 1
	cardImg.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(&cardImg)
	return err
}

// AddGymcardShare 新增游泳健身卡分享信息
func (m *GymcardDAO) AddGymcardShare(gymcardId, cardShareInfo, cardUseRule, cardUseInstraction string) error {
	var cardShare model.TaCardShare
	cardShare.CardShareId = utils.GetGUID()
	cardShare.GymCardId = gymcardId
	cardShare.CardShareInfo = cardShareInfo
	cardShare.CardUseInstruction = cardUseInstraction
	cardShare.CardUseRule = cardUseRule
	cardShare.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(&cardShare)
	return err
}

// DeleteGymcard 删除游泳健身卡
func (m *GymcardDAO) DeleteGymcard(gymcardId string) error {
	var gymcard = model.TaGymCard{
		GymCardId: gymcardId,
		Status:    models.STATUS_DEL,
	}
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", gymcard.GymCardId).Update(gymcard)
	return err
}

// DeleteCardImg 删除健身卡图片
func (m *GymcardDAO) DeleteCardImg(gymcardId string) error {
	var cardImg = model.TaCardImage{
		GymCardId: gymcardId,
		Status:    models.STATUS_DEL,
	}
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", cardImg.GymCardId).Update(cardImg)
	return err

}

// DeleteCardChannel 删除健身卡渠道对应
func (m *GymcardDAO) DeleteCardChannel(gymcardId string) error {
	var cardChannel = model.TaCardCouponChannel{
		CardCouponId: gymcardId,
		Status:       models.STATUS_DEL,
	}
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", cardChannel.CardCouponId).Update(cardChannel)
	return err
}

// DeleteCardImg 删除健身卡分享信息
func (m *GymcardDAO) DeleteCardShare(gymcardId string) error {
	var cardShare = model.TaCardShare{
		GymCardId: gymcardId,
		Status:    models.STATUS_DEL,
	}
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", cardShare.GymCardId).Update(cardShare)
	return err
}
func (m *GymcardDAO) EditGymcard(gymcard *model.TaGymCard) error {
	var cols = []string{
		"sent_count",
		"used_count",
		"status",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", gymcard.GymCardId).Update(gymcard)
	return err
}

// EditGymcardImg 修改健身卡图片
func (m *GymcardDAO) EditGymcardImg(gymcardId, imgUrl string) error {
	var cardImg = model.TaCardImage{
		GymCardId:    gymcardId,
		CardImageUrl: imgUrl,
	}
	var cols = []string{
		"card_image_url",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", cardImg.GymCardId).Update(cardImg)
	return err
}

// EditGymcardShare 修改健身卡分享信息
func (m *GymcardDAO) EditGymcardShare(gymcardId, cardShareInfo, cardUseRule, cardUseInstraction string) error {
	var cardShare = model.TaCardShare{
		GymCardId:          gymcardId,
		CardShareInfo:      cardShareInfo,
		CardUseRule:        cardUseRule,
		CardUseInstruction: cardUseInstraction,
	}
	var cols = []string{
		"card_share_info",
		"card_use_rule",
		"card_use_instruction",
	}
	_, err := m.db.Cols(cols...).Where("gym_card_id = ?", cardShare.GymCardId).Update(cardShare)
	return err
}

// SendGymcardToCustomer 新增用户健身卡
func (m *GymcardDAO) SendGymcardToCustomer(gymcard *model.TaGymCard, customerId, salesId, salesName, receivingtype string) (*model.TaCustomerGym, error) {
	var customerGym model.TaCustomerGym
	customerGym.CustomerGymId = utils.GetGUID()
	customerGym.CaseId = gymcard.CaseId
	customerGym.CustomerGymName = gymcard.GymCardName
	customerGym.CustomerId = customerId
	customerGym.EndDate = gymcard.EndDate
	customerGym.GymCardId = gymcard.GymCardId
	customerGym.GymType = gymcard.GymTypeId
	customerGym.OrgId = gymcard.OrgId
	customerGym.ReceiveDate = time.Now()
	customerGym.StartDate = gymcard.StartDate
	customerGym.Status = models.STATUS_NORMAL
	customerGym.SalesId = salesId
	customerGym.SalesName = salesName
	customerGym.CardStatus = models.CARD_COUPON_RECEIVED
	customerGym.ReceivingType = receivingtype
	_, err := m.db.Insert(customerGym)
	return &customerGym, err
}

// UpdateCustomerGym 修改用户卡
func (m *GymcardDAO) UpdateCustomerGym(customerGym model.TaCustomerGym) error {
	var cols = []string{
		"start_date",
		"end_date",
		"status",
	}
	_, err := m.db.Cols(cols...).Where("customer_gym_id =?", customerGym.CustomerGymId).Update(customerGym)
	return err
}

// AddGymcardUsage 新增用户健身卡使用记录
func (m *GymcardDAO) AddGymcardUsage(customerGym model.TaCustomerGym) error {
	var gymcardUsage model.TaGymCardUsage
	gymcardUsage.CustomerGymId = customerGym.CustomerGymId
	gymcardUsage.GymCardUsageId = utils.GetGUID()
	gymcardUsage.UseCaseId = customerGym.CaseId
	gymcardUsage.UseDate = time.Now()
	gymcardUsage.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(gymcardUsage)
	return err
}

// GetCustomerGymByCustomerId 通过用户id获取用户所有游泳健身卡
func (m *GymcardDAO) GetCustomerGymByCustomerId(customerId string, page, pageSize int) ([]CustomerGym, error) {
	var customerGym []CustomerGym
	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule,
	d.price
FROM
	ta_customer_gym a
INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
INNER JOIN ta_card_share C oN a.gym_card_id = c.gym_card_id
INNER JOIN ta_gym_card d ON a.gym_card_id = d.gym_card_id
WHERE
	a.customer_id = '` + customerId + `' and a.status >` + strconv.Itoa(models.STATUS_DEL)
	sql += ` order by a.receive_date desc limit ` + strconv.Itoa(page) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&customerGym)
	return customerGym, err
}

// GetCustomerGymByPhone 根据用户电话获取用户可用有用健身卡
func (m *GymcardDAO) GetCustomerGymByPhone(phone string) ([]CustomerGym, error) {
	var customerGym []CustomerGym
	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule,
	d.price
FROM
	ta_customer_gym a
INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
INNER JOIN ta_card_share C oN a.gym_card_id = c.gym_card_id
INNER JOIN ta_customer d ON a.customer_id = d.customer_id
INNER JOIN ta_gym_card e ON a.gym_card_id = e.gym_card_id
Where d.phone = '` + phone + `' and a.status >` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&customerGym)
	return customerGym, err
}

// GetCustomerGymCountByCustomerId 通过用户id获取用户所有游泳健身卡数量
func (m *GymcardDAO) GetCustomerGymCountByCustomerId(customerId string) (int, error) {
	var customerGym []CustomerGym
	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule
FROM
	ta_customer_gym a
INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
INNER JOIN ta_card_share C oN a.gym_card_id = c.gym_card_id
WHERE
	a.customer_id = '` + customerId + `' and a.status >` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&customerGym)
	return len(customerGym), err

}

// GetCustomerGymById 根据id获取用户健身卡详情
func (m *GymcardDAO) GetCustomerGymById(customerCardId string) (*CustomerGym, error) {
	var customerGym []CustomerGym
	sql := `SELECT
	a.*,
	b.card_image_url,
	c.card_share_info,
	c.card_use_instruction,
	c.card_use_rule,
	d.price
FROM
	ta_customer_gym a
INNER JOIN ta_card_image b ON a.gym_card_id = b.gym_card_id
INNER JOIN ta_card_share C oN a.gym_card_id = c.gym_card_id
INNER JOIN ta_gym_card d ON a.gym_card_id = d.gym_card_id
WHERE
	a.customer_gym_id = '` + customerCardId + `' and a.status >` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&customerGym)
	if err != nil {
		return nil, err
	}
	if len(customerGym) > 0 {
		return &customerGym[0], nil
	}
	return nil, err
}

// AddGiveRecord 新增赠送记录
func (m *GymcardDAO) AddGiveRecord(giveRecord *model.TaCouponGiveRecord) error {
	giveRecord.CreateDate = time.Now()
	giveRecord.RecId = utils.GetGUID()
	giveRecord.GiftType = models.GIVE_GIFT_GYM
	_, err := m.db.Insert(giveRecord)
	return err
}
