package statistics

import (
	"spaceofcheng/services/models"
	"strconv"
	"strings"
	"time"
)

// CourseOrdersStatistics 课程订单
type CourseOrdersStatistics struct {
	OrdersId     string
	OrdersNo     string
	CourseName   string
	CaseName     string
	LocationName string
	BeginDate    time.Time
	EndDate      time.Time
	Name         string
	CustomerName string
	Phone        string
	CreateDate   time.Time
}

// GetCourseOrdersStatistics 获取课程订单统计信息
func (m *StatisticsDAO) GetCourseOrdersStatistics(typeid, caseids, name, begindate, enddate string, page, pageSize int) ([]CourseOrdersStatistics, error) {
	var courseOrders []CourseOrdersStatistics
	sql := `SELECT
					a.orders_id,
					a.orders_no,
					c.course_name,
					b.case_name,
					d.location_name,
					c.begin_date,
					c.end_date,
					e. NAME,
					e.customer_name,
					e.phone,
					a.create_date
				FROM
					ta_course_orders a
				INNER JOIN sys_case b ON a.case_id = b.case_id
				INNER JOIN ta_course c ON a.course_id = c.course_id
				INNER JOIN td_cms_image_location d ON c.location_id = d.location_id
				INNER JOIN ta_customer e ON a.customer_id = e.customer_id 
				Where c.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`
	if typeid != "" {
		sql = sql + ` and c.location_id = '` + typeid + `'`
	}
	if name != "" {
		sql = sql + ` and c.course_name like '%` + name + `%'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	sql = sql + ` order by a.create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)

	err := m.db.Sql(sql).Find(&courseOrders)
	return courseOrders, err
}

// GetCourseOrdersStatisticsCount 获取课程订单统计数量
func (m *StatisticsDAO) GetCourseOrdersStatisticsCount(typeid, caseids, name, begindate, enddate string) ([]CourseOrdersStatistics, error) {
	var courseOrders []CourseOrdersStatistics
	sql := `SELECT
					a.orders_id,
					a.orders_no,
					c.course_name,
					b.case_name,
					d.location_name,
					c.begin_date,
					c.end_date,
					e. NAME,
					e.customer_name,
					e.phone,
					a.create_date
				FROM
					ta_course_orders a
				INNER JOIN sys_case b ON a.case_id = b.case_id
				INNER JOIN ta_course c ON a.course_id = c.course_id
				INNER JOIN td_cms_image_location d ON c.location_id = d.location_id
				INNER JOIN ta_customer e ON a.customer_id = e.customer_id 
				Where c.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`
	if typeid != "" {
		sql = sql + ` and c.location_id = '` + typeid + `'`
	}
	if name != "" {
		sql = sql + ` and c.course_name like '%` + name + `%'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	sql = sql + ` order by a.create_date desc `
	err := m.db.Sql(sql).Find(&courseOrders)
	return courseOrders, err
}

// CourseStatistics 课程统计数据
type CourseStatistics struct {
	CourseId     string
	CaseName     string
	CourseName   string
	LocationName string
	BeginDate    time.Time
	EndDate      time.Time
	Price        string
	Total        int
	JoinTotal    int
}

// GetCourseStatistics 获取课程统计信息
func (m *StatisticsDAO) GetCourseStatistics(typeid, caseids, name string, page, pageSize int) ([]CourseStatistics, error) {
	var courses []CourseStatistics
	sql := `SELECT
					a.course_id,
					d.case_name,
					a.course_name,
					e.location_name,
					a.begin_date,
					a.end_date,
					a.price,
					b.total,
					c.join_total
				FROM
					ta_course a
				INNER JOIN sys_case d ON a.case_id = d.case_id
				INNER JOIN td_cms_image_location e ON a.location_id = e.location_id
				LEFT JOIN (
					SELECT
						course_id,
						count(1) AS total
					FROM
						ta_customer_course
					GROUP BY
						course_id
				) b ON a.course_id = b.course_id
				LEFT JOIN (
					SELECT
						course_id,
						count(1) AS join_total
					FROM
						ta_customer_course
					WHERE
						join_num > 0
					GROUP BY
						course_id
				) c ON a.course_id = c.course_id
				Where a.status = ` + strconv.Itoa(models.STATUS_NORMAL) + ` and d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`
	if typeid != "" {
		sql = sql + ` and e.location_id = '` + typeid + `'`
	}
	if name != "" {
		sql = sql + ` and a.course_name like '%` + name + `%'`
	}

	sql = sql + ` order by a.create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)

	err := m.db.Sql(sql).Find(&courses)
	return courses, err
}

// GetCourseStatisticsCount 获取课程统计信息数量
func (m *StatisticsDAO) GetCourseStatisticsCount(typeid, caseids, name string) ([]CourseStatistics, error) {
	var courses []CourseStatistics
	sql := `SELECT
					a.course_id,
					d.case_name,
					a.course_name,
					e.location_name,
					a.begin_date,
					a.end_date,
					a.price
				FROM
					ta_course a
				INNER JOIN sys_case d ON a.case_id = d.case_id
				INNER JOIN td_cms_image_location e ON a.location_id = e.location_id
				Where a.status = ` + strconv.Itoa(models.STATUS_NORMAL) + ` and d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`
	if typeid != "" {
		sql = sql + ` and e.location_id = '` + typeid + `'`
	}
	if name != "" {
		sql = sql + ` and a.course_name like '%` + name + `%'`
	}
	sql = sql + ` order by a.create_date desc `
	err := m.db.Sql(sql).Find(&courses)
	return courses, err
}
