package statistics

import (
	"spaceofcheng/services/models"
	"strconv"
	"strings"
	"time"
)

var sqlThisWeek = `
SELECT
	DATE_FORMAT(
		date_add(
			date_sub(
				curdate(),
				INTERVAL WEEKDAY(curdate()) + 1 DAY
			),
			INTERVAL rownum DAY
		),
		'%Y-%m-%d'
	) AS dt
FROM sequence
WHERE rownum < 7
`

var sqlThisMonth = `
SELECT
	DATE_FORMAT(
		DATE_add(
			DATE_add(
				LAST_DAY(curdate() - INTERVAL 1 MONTH),
				INTERVAL 1 DAY
			),
			INTERVAL rownum DAY
		),
		'%Y-%m-%d'
	) AS dt
FROM
	sequence
WHERE
	rownum < (
		SELECT
			DATEDIFF(
				date_add(
					curdate() - DAY (curdate()) + 1,
					INTERVAL 1 MONTH
				),
				DATE_ADD(
					curdate(),
					INTERVAL - DAY (curdate()) + 1 DAY
				)
			)
	)
`

// GetCustomerNumber 获取会员数量
func (m *StatisticsDAO) GetCustomerNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_customer", orgID, caseIDs, start, end)
}

// GetCourseOrderNumber 获取课程预约 - 统计课程
func (m *StatisticsDAO) GetCourseOrderNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_course_order", orgID, caseIDs, start, end)
}

// GetCourseOrderCustomer 获取课程预约 - 统计人员
func (m *StatisticsDAO) GetCourseOrderCustomer(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_course_order_customer", orgID, caseIDs, start, end)
}

// GetUsedCourseOrderCustomer 获取课程核销 - 统计人员
func (m *StatisticsDAO) GetUsedCourseOrderCustomer(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_used_course_order_customer", orgID, caseIDs, start, end)
}

// GetOrderGoodsNumber 获取商品下单数量
func (m *StatisticsDAO) GetOrderGoodsNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {

	return m.execMySQLFunc("fun_order_goods_num", orgID, caseIDs, start, end)
}

// GetCourseNumber 获取课程数量
func (m *StatisticsDAO) GetCourseNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_course", orgID, caseIDs, start, end)
}

// GetArrivedCustomerNumber 获取到场人数
func (m *StatisticsDAO) GetArrivedCustomerNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_arrived_customer", orgID, caseIDs, start, end)
}

// GetCourseCardCustomer 获取预约人数 - 含健身卡
func (m *StatisticsDAO) GetCourseCardCustomer(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_course_card_customer", orgID, caseIDs, start, end)
}

// GetCouponCardNumber 获取卡券总数
func (m *StatisticsDAO) GetCouponCardNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_coupon_card", orgID, caseIDs, start, end)
}

// GetUsedCouponCardNumber 获取已使用卡券
func (m *StatisticsDAO) GetUsedCouponCardNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_used_coupon_card", orgID, caseIDs, start, end)
}

// GetCustomerCouponCardNumber 获取已发放卡券总数
func (m *StatisticsDAO) GetCustomerCouponCardNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_customer_coupon_card", orgID, caseIDs, start, end)
}

// StaCommData 折线图 或 柱图类数据
type StaCommData struct {
	Dt     string `json:"x"`
	Amount int    `json:"y"`
}

// StaCommGroupData 折线图 或 柱图类数据 含分组
type StaCommGroupData struct {
	Dt     string `json:"x"`
	Amount int    `json:"y"`
	Group  string `json:"g"`
}

// StaPieData 饼图类数据
type StaPieData struct {
	Item    string  `json:"item"`
	Amount  int     `json:"count"`
	Percent float64 `json:"percent"`
}

// GetCourseOrderPie 获取预约课程
func (m *StatisticsDAO) GetCourseOrderPie(orgID string, caseIDs []string, start, end *time.Time) ([]StaPieData, error) {
	sql := `
		SELECT
			t.course_name AS item,
			count(DISTINCT course_id) AS amount,
			0.0 AS percent
		FROM
			ta_customer_course_detail t
		WHERE
			t.status = ?
			AND t.org_id = ?
	`
	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND t.case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	startDate := ""
	endDate := ""

	if start != nil {
		startDate = start.Format("2006-01-02 15:04:05")
	}
	if end != nil {
		endDate = end.Format("2006-01-02 15:04:05")
	}

	if startDate != "" {
		sql += ` AND DATE_FORMAT(t.start_date, '%Y-%m-%d %T') >= '` + startDate + `'`
	}

	if endDate != "" {
		sql += ` AND DATE_FORMAT(t.start_date, '%Y-%m-%d %T') <= '` + endDate + `'`
	}

	sql += `		
		GROUP BY
			t.course_name
	`

	var data []StaPieData
	if err := m.db.SQL(sql, models.STATUS_NORMAL, orgID).Find(&data); err != nil {
		return nil, err
	}

	if data == nil || len(data) == 0 {
		return nil, nil
	}

	total := 0
	for _, dt := range data {
		total += dt.Amount
	}

	if total != 0 {
		for i := range data {
			data[i].Percent = float64(data[i].Amount / total)
		}
	}

	return data, nil
}

// GetNewCustomerDaily 按天统计新增会员
func (m *StatisticsDAO) GetNewCustomerDaily(orgID string, caseIDs []string, staType string) ([]StaCommData, error) {
	sql := `
		SELECT
			t.dt,
			IFNULL(s.amount, 0) AS amount
		FROM (
	`

	if staType == "week" {
		sql += sqlThisWeek
	} else {
		sql += sqlThisMonth
	}

	sql += `) t
		LEFT JOIN ta_sta_customer_daily s ON t.dt = s.sta_date
		AND s.org_id = '` + orgID + `'
	`

	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	var res []StaCommData
	if err := m.db.SQL(sql).Find(&res); err != nil {
		return nil, err
	}

	return res, nil
}

// GetCourseDaily 按天统计排课数
func (m *StatisticsDAO) GetCourseDaily(orgID string, caseIDs []string, staType string) ([]StaCommData, error) {
	sql := `
		SELECT
			t.dt,
			IFNULL(s.amount, 0) AS amount
		FROM (
	`

	if staType == "week" {
		sql += sqlThisWeek
	} else {
		sql += sqlThisMonth
	}

	sql += `) t
		LEFT JOIN ta_sta_course_daily s ON t.dt = s.sta_date
		AND s.org_id = '` + orgID + `'
	`

	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	var res []StaCommData
	if err := m.db.SQL(sql).Find(&res); err != nil {
		return nil, err
	}

	return res, nil
}

// GetCourseOrderDaily 按天统计课程预约数
func (m *StatisticsDAO) GetCourseOrderDaily(orgID string, caseIDs []string, staType string) ([]StaCommData, error) {
	sql := `
		SELECT
			t.dt,
			sum(IFNULL(s.amount, 0)) AS amount
		FROM (
	`

	if staType == "week" {
		sql += sqlThisWeek
	} else {
		sql += sqlThisMonth
	}

	sql += `) t
		LEFT JOIN ta_sta_course_order_daily s ON t.dt = s.sta_date
		AND s.org_id = '` + orgID + `'
	`

	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	sql += ` group by t.dt`

	var res []StaCommData
	if err := m.db.SQL(sql).Find(&res); err != nil {
		return nil, err
	}

	return res, nil
}

// GetGoodsDaily 按天统计饮品下单数
func (m *StatisticsDAO) GetGoodsDaily(orgID string, caseIDs []string, staType string) ([]StaCommData, error) {
	sql := `
		SELECT
			t.dt,
			sum(IFNULL(s.amount, 0)) AS amount
		FROM (
	`

	if staType == "week" {
		sql += sqlThisWeek
	} else {
		sql += sqlThisMonth
	}

	sql += `) t
		LEFT JOIN ta_sta_goods_daily s ON t.dt = s.sta_date
		AND s.org_id = '` + orgID + `'
	`

	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	sql += ` group by t.dt`

	var res []StaCommData
	if err := m.db.SQL(sql).Find(&res); err != nil {
		return nil, err
	}

	return res, nil
}

// GetGoodsWithTypeDaily 按天分组统计饮品下单数
func (m *StatisticsDAO) GetGoodsWithTypeDaily(orgID string, caseIDs []string) ([]StaCommGroupData, error) {
	sql := `
		SELECT
			t.dt,
			s.type_name as 'group',
			sum(IFNULL(s.amount, 0)) AS amount
		FROM (
	`

	sql += sqlThisMonth + `) t
		LEFT JOIN ta_sta_goods_daily s ON t.dt = s.sta_date
		AND s.org_id = '` + orgID + `'
	`

	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	sql += ` group by t.dt, s.type_name`
	var res []StaCommGroupData
	if err := m.db.SQL(sql).Find(&res); err != nil {
		return nil, err
	}

	return res, nil
}

// GetArrivedDaily 按天统计到场人次
func (m *StatisticsDAO) GetArrivedDaily(orgID string, caseIDs []string, staType string) ([]StaCommData, error) {
	sql := `
		SELECT
			t.dt,
			IFNULL(s.amount, 0) AS amount
		FROM (
	`

	if staType == "week" {
		sql += sqlThisWeek
	} else {
		sql += sqlThisMonth
	}

	sql += `) t
		LEFT JOIN ta_sta_arrived_daily s ON t.dt = s.sta_date
		AND s.org_id = '` + orgID + `'
	`

	if caseIDs != nil && len(caseIDs) > 0 {
		sql += ` AND case_id in ('` + strings.Join(caseIDs, "','") + `')`
	}

	sql += ` group by t.dt`

	var res []StaCommData
	if err := m.db.SQL(sql).Find(&res); err != nil {
		return nil, err
	}

	return res, nil
}

// execMySQLFunc 执行 mysql 函数
func (m *StatisticsDAO) execMySQLFunc(funName, orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	csids := ""
	if caseIDs != nil && len(caseIDs) > 0 {
		csids = strings.Join(caseIDs, ",")
	}

	startDate := ""
	endDate := ""

	if start != nil {
		startDate = start.Format("2006-01-02 15:04:05")
	}
	if end != nil {
		endDate = end.Format("2006-01-02 15:04:05")
	}

	sql := "select " + funName + "(?,?,?,?) as total"
	res, err := m.db.Query(sql, orgID, csids, startDate, endDate)
	if err != nil {
		return 0, err
	}

	if res == nil || len(res) == 0 {
		return 0, nil
	}

	total, _ := strconv.ParseInt(string(res[0]["total"]), 10, 64)
	return total, nil
}
