package statistics

import (
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"
)

// GoodsStatistics 商品统计信息
type GoodsStatistics struct {
	CaseName    string
	GoodsName   string
	TypeName    string
	Price       string
	Total       int
	FinishTotal int
	Amount      string
}

// GetGoodsStatistics 获取商品统计信息
func (m *StatisticsDAO) GetGoodsStatistics(caseids, typeid, name string, page, pageSize int) ([]GoodsStatistics, error) {
	var goods []GoodsStatistics
	sql := `SELECT
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price,
					sum(a.number) AS total,
					sum(
						(
							CASE
							WHEN d.make_status = 'finished' THEN
								1
							ELSE
								0
							END
						)
					) AS finish_total,
					b.price * sum(a.number) AS amount
				FROM
					ta_goods_orders_detail a
				INNER JOIN ta_goods b ON a.goods_id = b.goods_id
				INNER JOIN td_goods_type c ON b.type_id = c.type_id
				INNER JOIN ta_goods_orders d ON a.orders_id = d.orders_id
				inner join sys_case e on b.case_id=e.case_id 
				where e.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if typeid != "" {
		sql = sql + ` and c.type_id='` + typeid + `'`
	}

	if name != "" {
		sql = sql + ` and b.goods_name like '%` + name + `%'`
	}
	sql = sql + ` GROUP BY
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&goods)
	return goods, err
}

// GetGoodsStatisticsCount 获取总数
func (m *StatisticsDAO) GetGoodsStatisticsCount(caseids, typeid, name string) ([]GoodsStatistics, error) {
	var goods []GoodsStatistics
	sql := `SELECT
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price,
					sum(a.number) AS total,
					sum(
						(
							CASE
							WHEN d.make_status = 'finished' THEN
								1
							ELSE
								0
							END
						)
					) AS finish_total,
					b.price * sum(a.number) AS amount
				FROM
					ta_goods_orders_detail a
				INNER JOIN ta_goods b ON a.goods_id = b.goods_id
				INNER JOIN td_goods_type c ON b.type_id = c.type_id
				INNER JOIN ta_goods_orders d ON a.orders_id = d.orders_id
				inner join sys_case e on b.case_id=e.case_id 
				where e.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if typeid != "" {
		sql = sql + ` and c.type_id='` + typeid + `'`
	}

	if name != "" {
		sql = sql + ` and b.goods_name like '%` + name + `%'`
	}
	sql = sql + ` GROUP BY
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price`
	err := m.db.Sql(sql).Find(&goods)
	return goods, err
}

// GoodsOrdersStatistics 商品订单统计信息
type GoodsOrdersStatistics struct {
	OrdersId      string
	OrdersNo      string
	CaseName      string
	UserType      string
	CreateDate    time.Time
	Status        int
	MakeStatus    string
	AreaName      string
	TableNo       string
	CustomerName  string
	Phone         string
	Name          string
	Remark        string
	AreaIconWhite string
	PayType       string
	PayTypeStr    string
	Details       []model.TaGoodsOrdersDetail
}

// GetGoodsOrderStatistics 获取商品订单
func (m *StatisticsDAO) GetGoodsOrderStatistics(status, usertype, caseids, begindate, enddate, paytype string, page, pageSize int) ([]GoodsOrdersStatistics, int64, error) {
	var orders []GoodsOrdersStatistics
	sql := `SELECT
						a.orders_id,
						a.orders_no,
						d.case_name,
						a.user_type,
						a.create_date,
						a. STATUS,
						a.make_status,
						a.area_name,
						a.table_no,
						b.customer_name,
						e.area_icon_white,
						(
							CASE
							WHEN c.real_name IS NULL
							OR c.real_name = '' THEN
								b. NAME
							ELSE
								c.real_name
							END
						) AS NAME,
						b.phone,
						a.remark,
						a.pay_type
					FROM
						ta_goods_orders a
					INNER JOIN ta_customer b ON a.user_id = b.customer_id
					INNER JOIN sys_case d ON a.case_id = d.case_id
					LEFT JOIN sys_user c ON b.user_id = c.user_id
					LEFT JOIN sys_case_area e ON a.area_id =e.area_id
					where d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if usertype != "" {
		sql = sql + ` and a.user_type='` + usertype + `'`
	}
	if paytype != "" {
		sql = sql + ` and a.pay_type='` + paytype + `'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	if status != "" {
		sql = sql + ` and a.make_status='` + status + `'`
	}
	sql = sql + ` order by a.create_date desc`

	limit := []int{
		pageSize,
		(page - 1) * pageSize,
	}

	total, err := utils.NewPageNaviEngine(m.ctx).GetPageList(&orders, sql, limit)
	if err != nil {
		return nil, 0, err
	}
	for inx, item := range orders {
		details, err := m.GetGoodsOrderDetailByOrderId(item.OrdersId)
		if err != nil {
			return nil, 0, err
		}
		orders[inx].Details = details
	}
	return orders, total, err
}

// GetGoodsOrderDetailByOrderId 获取商品详情
func (m *StatisticsDAO) GetGoodsOrderDetailByOrderId(ordersid string) ([]model.TaGoodsOrdersDetail, error) {
	var details []model.TaGoodsOrdersDetail
	err := m.db.Where("orders_id=?", ordersid).Find(&details)
	return details, err
}

// GetGoodsOrderStatisticsCount 获取商品订单Count
func (m *StatisticsDAO) GetGoodsOrderStatisticsCount(status, usertype, caseids, begindate, enddate, paytype string) ([]GoodsOrdersStatistics, error) {
	var orders []GoodsOrdersStatistics
	sql := `SELECT
						a.orders_id,
						a.orders_no,
						d.case_name,
						a.user_type,
						a.create_date,
						a. STATUS,
						a.make_status,
						a.area_name,
						a.table_no,
						b.customer_name,
						(
							CASE
							WHEN c.real_name IS NULL
							OR c.real_name = '' THEN
								b. NAME
							ELSE
								c.real_name
							END
						) AS NAME,
						b.phone,
						a.remark
					FROM
						ta_goods_orders a
					INNER JOIN ta_customer b ON a.user_id = b.customer_id
					INNER JOIN sys_case d ON a.case_id = d.case_id
					LEFT JOIN sys_user c ON b.user_id = c.user_id
					where d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if usertype != "" {
		sql = sql + ` and a.user_type='` + usertype + `'`
	}
	if paytype != "" {
		sql = sql + ` and a.pay_type='` + paytype + `'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	if status != "" {
		sql = sql + ` and a.make_status='` + status + `'`
	}
	sql += ` order by a.create_date desc `
	err := m.db.Sql(sql).Find(&orders)
	return orders, err
}
