package statistics

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"strings"
)

// GetUserTypes 获取人员类型
func (m *StatisticsDAO) GetUserTypes(id string) ([]model.SysUserType, error) {
	if id == "" {
		return nil, nil
	}

	var tps []model.SysUserType
	if err := m.db.Where("user_id=?", id).Find(&tps); err != nil {
		return nil, err
	}

	return tps, nil
}

type UserDashSetting struct {
	CompId   string
	CompName string
	Group    int
	CompType string
	TypeName string
}

// GetDashboardSetting 获取人员 dashboard 设置
func (m *StatisticsDAO) GetDashboardSetting(userID string, userTypes []string) ([]UserDashSetting, error) {
	filters := make([]string, 0)
	if userID != "" {
		filters = append(filters, "(t.for_type='"+models.DASH_SETTING_FOR_USER+"' AND t.owner='"+userID+"')")
	}

	if userTypes != nil {
		filters = append(filters, "(t.for_type='"+models.DASH_SETTING_FOR_USERTYPE+"' AND t.owner in ('"+strings.Join(userTypes, "','")+"'))")
	}

	if len(filters) == 0 {
		return nil, nil
	}

	sql := `
		SELECT DISTINCT
			t.comp_id,
			t.group,
			s.comp_name,
			s.comp_type,
			s.type_name
		FROM
			ta_dashboard_setting t
		JOIN td_statistics_comps s ON t.comp_id = s.comp_id
		where (` + strings.Join(filters, " OR ") + ") AND t.status = ? order by t.group asc, t.sort_no asc"

	var settings []UserDashSetting
	if err := m.db.SQL(sql, models.STATUS_NORMAL).Find(&settings); err != nil {
		return nil, err
	}

	return settings, nil
}

// GetAllFrontComponents 获取所有前端支持组件
func (m *StatisticsDAO) GetAllFrontComponents() ([]model.TdStatisticsComps, error) {
	var comps []model.TdStatisticsComps
	if err := m.db.Find(&comps); err != nil {
		return nil, err
	}

	return comps, nil
}
