package system

import (
	"errors"
	"math/rand"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/builder"
	"github.com/go-xorm/xorm"
	"github.com/yl10/kit/encrypt"
)

// UserDAO 当前数据库操作对象
type UserDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewUserDAO New Inst
func NewUserDAO(ctx *utils.Context) *UserDAO {
	return &UserDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

type UserListInfo struct {
	model.SysUser `xorm:"extends"`
	UserTypeNames string
}
type CustomerInfo struct {
	model.TaCustomer `xorm:"extends"`
	Remark           string
}

// GetUserList 用户列表
func (m *UserDAO) GetUserList(username, typeid, caseids string, page int, pageSize int) ([]UserListInfo, error) {
	var users []UserListInfo
	sql := `select user.*,c.user_type_names from sys_user user left join (
		select a.user_id,GROUP_CONCAT(b.type_name) as user_type_names from sys_user_type a
		INNER join td_user_type b on a.type_id = b.type_id
		group by a.user_id) c on user.user_id = c.user_id where user.user_id != '` + models.ADMIN_ID + `' and
		(status> ` + strconv.Itoa(models.STATUS_DEL) + ` and user_name like '%` + username + `%' and user.user_id in 
		(select user_id from sys_user_case where is_belong=1 and case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')))`

	if typeid != "" {
		sql = sql + " and user.user_id in (select user_id from sys_user_type where type_id = '" + typeid + "')"
	}

	sql = sql + " order by create_date desc limit " + strconv.Itoa((page-1)*pageSize) + ", " + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&users)
	return users, err
}

// GetUserCount 用户列表count
func (m *UserDAO) GetUserCount(username, typeid, caseids string) (int, error) {
	var users []UserListInfo
	sql := `select user.*,c.user_type_names from sys_user user left join (
		select a.user_id,GROUP_CONCAT(b.type_name) as user_type_names from sys_user_type a
		INNER join td_user_type b on a.type_id = b.type_id
		group by a.user_id) c on user.user_id = c.user_id where user.user_id != '` + models.ADMIN_ID + `' and
		(status> ` + strconv.Itoa(models.STATUS_DEL) + ` and user_name like '%` + username + `%' and user.user_id in 
		(select user_id from sys_user_case where is_belong=1 and case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')))`

	if typeid != "" {
		sql = sql + " and user.user_id in (select user_id from sys_user_type where type_id = '" + typeid + "')"
	}

	err := m.db.Sql(sql).Find(&users)
	if err != nil {
		return 0, err
	}
	return len(users), nil
}

// UserInfo 用户详情
type UserInfo struct {
	model.SysUser `xorm:"extends"`
	UserType      []model.TdUserType
	UserTag       []model.SysTag
	UserCase      []model.SysUserCase
	UserRole      []model.SysRole
	UserMenu      []model.SysMenu
}

// SysUserForm 映射用户
type SysUserForm struct {
	model.SysUser `xorm:"extends"`
	TypeId        string
	TagIds        string
	BelongCaseId  string
}

// GetUserByID 根据ID获取用户信息
func (m *UserDAO) GetUserByID(userid string) (*UserInfo, error) {
	var userinfo []UserInfo
	sql := `select * from sys_user where user_id='` + userid + `' and status>` + strconv.Itoa(models.STATUS_DEL)
	err := m.db.Sql(sql).Find(&userinfo)
	if err != nil {
		return nil, err
	}
	if len(userinfo) > 0 {
		return &userinfo[0], nil
	}
	return nil, nil
}

// GetUserTypes 获取所有的用户类型
func (m *UserDAO) GetUserTypes(orgid string) ([]model.TdUserType, error) {
	var usertype []model.TdUserType
	err := m.db.Where("status>"+strconv.Itoa(models.STATUS_DEL)).And("org_id=?", orgid).Find(&usertype)
	return usertype, err
}

// GetUserType 获取用户类型
func (m *UserDAO) GetUserType(userid string) ([]model.TdUserType, error) {
	var usertype []model.TdUserType
	err := m.db.In("type_id", builder.Select("type_id").From("sys_user_type").Where(builder.Eq{"user_id": userid}).And(builder.Gt{"status": models.STATUS_DEL})).Find(&usertype)
	return usertype, err
}

// GetUserTag 获取用户标签
func (m *UserDAO) GetUserTag(userid string) ([]model.SysTag, error) {
	var userTag []model.SysTag
	err := m.db.In("tag_id", builder.Select("tag_id").From("sys_user_tag").Where(builder.Eq{"user_id": userid}).And(builder.Gt{"status": models.STATUS_DEL})).Find(&userTag)
	return userTag, err
}

// GetUserCase 获取用户案场
func (m *UserDAO) GetUserCase(userid string) ([]model.SysUserCase, error) {
	var userCase []model.SysUserCase
	// err := m.db.Where("user_id=?", userid).And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&userCase)
	err := m.db.In("case_id", builder.Select("case_id").From("sys_user_case").Where(builder.Eq{"user_id": userid}).And(builder.Gt{"status": models.STATUS_DEL})).And(builder.Eq{"user_id": userid}).Find(&userCase)
	return userCase, err
}

// GetUserRole 获取用户角色
func (m *UserDAO) GetUserRole(userid string) ([]model.SysRole, error) {
	var userRole []model.SysRole
	err := m.db.In("role_id", builder.Select("role_id").From("sys_user_role").Where(builder.Eq{"user_id": userid}).And(builder.Gt{"status": models.STATUS_DEL})).Find(&userRole)
	return userRole, err
}

// GetUserMenu 获取用户菜单
func (m *UserDAO) GetUserMenu(userid string) ([]model.SysMenu, error) {
	type Menus struct {
		MenuIds string
	}
	var menu = Menus{}
	sql := `select group_concat(menu_id) as menu_ids from sys_user_menu where user_id = ?`
	m.db.Sql(sql, userid).Get(&menu)
	var userMenu []model.SysMenu
	var err error
	if menu.MenuIds != "" {
		sql = `select * from (select * from sys_menu where menu_id in ('` + strings.Replace(menu.MenuIds, ",", "','", -1) + `')
		UNION
		select * from sys_menu where menu_id in (select father_id from sys_menu where menu_id in ('` + strings.Replace(menu.MenuIds, ",", "','", -1) + `'))) s where status>? order by order_no asc
		`
		err = m.db.Sql(sql, models.STATUS_DEL).Find(&userMenu)
	}
	return userMenu, err
}

// AddUser 新增用户
func (m *UserDAO) AddUser(user model.SysUser) (*model.SysUser, error) {
	user.UserId = utils.GetGUID()
	user.CreateDate = time.Now()
	user.Pwd = encrypt.Md5(encrypt.Md5("123456"), user.UserId)
	user.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(user)
	return &user, err
}

func (m *UserDAO) GenerateRecommendCode() string {
	var temp1 int = rand.Intn(9) * 87
	var temp2 int = rand.Intn(9)
	var code string = strconv.Itoa(temp1) + strconv.Itoa(temp2)
	return code
}
func (m *UserDAO) IsCodeExist(code string) (int, error) {
	var user []model.SysUser
	sql := `select * from sys_user where recommend_code = '` + code + `'`
	err := m.db.Sql(sql).Find(&user)
	return len(user), err

}

// UpdateUser 修改用户
func (m *UserDAO) UpdateUser(user model.SysUser) error {
	var col = []string{
		"user_name",
		"real_name",
		"sex",
		"work_no",
		"email",
		"phone",
		"headimgurl",
		"user_type_name",
		"case_id",
	}
	_, err := m.db.Cols(col...).Where("user_id=?", user.UserId).Update(user)
	return err
}

// DelUser 删除用户
func (m *UserDAO) DelUser(userid string) error {
	var user = model.SysUser{}
	user.Status = models.STATUS_DEL
	user.UserId = userid
	var cols = []string{
		"status",
	}
	_, err := m.db.Cols(cols...).Where("user_id=?", userid).Update(user)
	return err
}

// UnleashCustomerUser 解除用户关联
func (m *UserDAO) UnleashCustomerUser(userid string) error {
	sql := `update ta_customer set user_id='' where user_id='` + userid + `'`
	_, err := m.db.Exec(sql)
	return err
}

// DelUserType 删除用户对应的类型信息
func (m *UserDAO) DelUserType(userid string) error {
	sql := "delete from sys_user_type where user_id='" + userid + "'"
	_, err := m.db.Exec(sql)
	return err
}

// SaveUserType 保存用户对应类型信息
func (m *UserDAO) SaveUserType(userid, typeids string) error {
	sql := `insert into sys_user_type(user_id,type_id) select '` + userid + `', type_id from td_user_type where type_id in ('` + strings.Replace(typeids, ",", "','", -1) + `') and status>` + strconv.Itoa(models.STATUS_DEL)
	_, err := m.db.Exec(sql)
	return err
}

// DelUserRole 删除用户对应的角色信息
func (m *UserDAO) DelUserRole(userid string) error {
	sql := "delete from sys_user_role where user_id='" + userid + "'"
	_, err := m.db.Exec(sql)
	return err
}

// SaveUserRole 保存用户对应角色信息
func (m *UserDAO) SaveUserRole(userid, roleids string) error {
	sql := `insert into sys_user_role(user_id,role_id) select '` + userid + `', role_id from sys_role where role_id in ('` + strings.Replace(roleids, ",", "','", -1) + `') and status>` + strconv.Itoa(models.STATUS_DEL)
	_, err := m.db.Exec(sql)
	return err
}

// GetUserByRole 根据角色获取用户信息
func (m *UserDAO) GetUserByRole(roleid string) ([]model.SysUserRole, error) {
	var userroles []model.SysUserRole
	err := m.db.Where("role_id=?", roleid).Find(&userroles)
	return userroles, err
}

// DelUserMenu 删除用户对应的菜单信息
func (m *UserDAO) DelUserMenu(userid string) error {
	sql := "delete from sys_user_menu where user_id='" + userid + "'"
	_, err := m.db.Exec(sql)
	return err
}

// SaveUserMenu 保存用户对应的菜单信息
func (m *UserDAO) SaveUserMenu(userid string) error {
	sql := `insert into sys_user_menu(user_id,menu_id) select DISTINCT '` + userid + `', a.menu_id from sys_role_menu a inner join sys_user_role b on a.role_id = b.role_id inner join sys_menu c on c.menu_id = a.menu_id where b.user_id = '` + userid + `' and c.status>` + strconv.Itoa(models.STATUS_DEL)
	_, err := m.db.Exec(sql)
	return err
}

// DelUserTag 删除用户对应的标签信息
func (m *UserDAO) DelUserTag(userid string) error {
	sql := "delete from sys_user_tag where user_id='" + userid + "'"
	_, err := m.db.Exec(sql)
	return err
}

// SaveUserTag 保存用户对应标签信息
func (m *UserDAO) SaveUserTag(userid, tagids string) error {
	sql := `insert into sys_user_tag(user_id,tag_id) select '` + userid + `', tag_id from sys_tag where tag_id in ('` + strings.Replace(tagids, ",", "','", -1) + `') and status>` + strconv.Itoa(models.STATUS_DEL)
	_, err := m.db.Exec(sql)
	return err
}

// GetUserBelongCase 获取用户默认案场
func (m *UserDAO) GetUserBelongCase(userid string) (model.SysUserCase, error) {
	var usercase model.SysUserCase
	_, err := m.db.Where("is_belong=1").And("user_id=?", userid).Get(&usercase)
	return usercase, err
}

// DelUserBelongCase 删除用户的所属案场
func (m *UserDAO) DelUserBelongCase(userid string) error {
	sql := "delete from sys_user_case where is_belong=1 and is_created=0 and user_id='" + userid + "'"
	_, err := m.db.Exec(sql)
	if err != nil {
		return err
	}
	sql = "update sys_user_case set is_belong=0 where user_id='" + userid + "' and is_created=1"
	_, err = m.db.Exec(sql)
	return err
}

// GetUserCreatedCaseHas 获取用户创建案场是否存在某案场信息
func (m *UserDAO) GetUserCreatedCaseHas(userid, caseid string) (bool, error) {
	var usercase []model.SysUserCase
	sql := `select * from sys_user_case where is_created = 1 and user_id=? and case_id=?`
	err := m.db.Sql(sql, userid, caseid).Find(&usercase)
	if err != nil || len(usercase) == 0 {
		return false, err
	}
	return true, nil
}

// DelUserCase 删除用户的案场信息
func (m *UserDAO) DelUserCase(userid string) error {
	sql := "delete from sys_user_case where (is_created=0 or is_created is null) and user_id='" + userid + "'"
	_, err := m.db.Exec(sql)
	if err != nil {
		return err
	}
	sql = "update sys_user_case set is_belong=0 where user_id='" + userid + "' and is_created=1"
	_, err = m.db.Exec(sql)
	return err
}

// SaveUserCase 保存用户对应案场信息
func (m *UserDAO) SaveUserCase(usercase model.SysUserCase) error {
	usercase.CreateDate = time.Now()
	usercase.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(usercase)
	return err
}

// UpdateUserCaseBelong 更新用户所属案场
func (m *UserDAO) UpdateUserCaseBelong(userid, caseid string) error {
	sql := `update sys_user_case set is_belong=1 where user_id=? and case_id=?`
	_, err := m.db.Exec(sql, userid, caseid)
	return err
}

// SaveUserCaseByUserTag 根据用户标签保存用户对应案场
func (m *UserDAO) SaveUserCaseByUserTag(userid, belongCaseId string) error {
	nowtime := time.Now().Local()
	sql := `insert into sys_user_case(user_id, case_id, case_name, is_belong, status, create_date, is_created) 
	select DISTINCT c.user_id, a.case_id,a.case_name, 0,?,?,0 from sys_case a 
	inner join sys_tag_case b on a.case_id = b.case_id
	inner join sys_user_tag c on b.tag_id = c.tag_id
	where c.user_id = ? and a.status > ` + strconv.Itoa(models.STATUS_DEL) + ` and a.case_id <> ?`

	_, err := m.db.Exec(sql, models.STATUS_NORMAL, nowtime, userid, belongCaseId)
	return err
}

// ValidatePassword 验证密码是否正确
func (m *UserDAO) ValidatePassword(user *model.SysUser, password string) bool {
	return user.Pwd == encrypt.Md5(password, user.UserId)
}

// UpdatePassword 更新用户密码
func (m *UserDAO) UpdatePassword(userID, newPass string) error {
	password := encrypt.Md5(newPass, userID)

	sql := "update sys_user set pwd=? where user_id=?"

	_, err := m.db.Exec(sql, password, userID)
	if err != nil {
		return err
	}

	return nil
}

// SaveUserMapping 保存用户映射信息
func (m *UserDAO) SaveUserMapping(usermapping *model.TaUserMapping) error {
	usermapping.MappingId = utils.GetGUID()
	_, err := m.db.Insert(usermapping)
	return err
}

// UpdateUserHeadImg 修改用户头像
func (m *UserDAO) UpdateUserHeadImg(userid, imgurl string) error {
	var col = []string{
		"headimgurl",
	}
	_, err := m.db.Cols(col...).Where("user_id=?", userid).Update(model.SysUser{
		Headimgurl: imgurl,
	})
	return err
}

// UpdateCustomerHeadImg 更新会员头像
func (m *UserDAO) UpdateCustomerHeadImg(customerid, username, imgurl string) error {
	var col = []string{
		"headimgurl",
	}
	_, err := m.db.Cols(col...).Where("customer_id=?", customerid).Update(model.TaCustomer{
		Headimgurl: imgurl,
	})
	return err
}

// GetUserCustomer 获取我的推荐客户
func (m *UserDAO) GetUserCustomer(userid, isrecommend, key string, page int, pageSize int) ([]CustomerInfo, error) {
	var customers []CustomerInfo
	sql := `select a.*,
	( SELECT remark FROM ta_sale_customer_remark b WHERE a.customer_id = b.customer_id ORDER BY b.create_date DESC LIMIT 1 ) AS remark from ta_customer a where (a.recommend_id='` + userid + `'`
	if isrecommend == "" || isrecommend == "false" {
		sql += ` or a.customer_id in (
			select customer_id from ta_customer_coupon where sales_id='` + userid + `'
			union 
			select customer_id from ta_customer_card where sales_id='` + userid + `'
		)`
	}
	sql += ")"
	if key != "" {
		sql += ` and (a.customer_name like '%` + key + `%' or a.phone like '%` + key + `%' or a.name like '%` + key + `%')`
	}
	offset := (page - 1) * pageSize
	sql += ` order by a.create_date desc LIMIT ` + strconv.Itoa(pageSize) + ` OFFSET ` + strconv.Itoa(offset)
	err := m.db.Sql(sql).Find(&customers)
	return customers, err
}

// GetUserByName 依据账户获取人员信息
func (m *UserDAO) GetUserByName(userName string) (*model.SysUser, error) {
	user := model.SysUser{}

	has, err := m.db.Where("user_name=?", userName).And("status>?", models.STATUS_DEL).Get(&user)
	if err != nil {
		return nil, err
	}

	if !has {
		return nil, nil
	}

	return &user, nil
}

// GetUserByPhone 依据手机号获取人员信息
func (m *UserDAO) GetUserByPhone(phone string) (*model.SysUser, error) {
	user := model.SysUser{}

	has, err := m.db.Where("phone=?", phone).And("status>?", models.STATUS_DEL).Get(&user)
	if err != nil {
		return nil, err
	}

	if !has {
		return nil, nil
	}

	return &user, nil
}

// UpdateUserMapping 更新用户映射
func (m *UserDAO) UpdateUserMapping(userMap *model.TaUserMapping, cols []string) error {
	if userMap.MappingId == "" {
		return errors.New("用户映射信息不存在")
	}

	_, err := m.db.Where("mapping_id=?", userMap.MappingId).Cols(cols...).Update(userMap)
	return err
}

type UserWithType struct {
	model.SysUser `xorm:"extends"`
	TypeId        string
	TypeName      string
}

// GetUserWithTypeByID 获取用户类型
func (m *UserDAO) GetUserWithTypeByID(userID string) ([]UserWithType, error) {
	query := `
		SELECT
			t.*, m.type_id,
			m.type_name
		FROM
			sys_user t
		JOIN sys_user_type s ON t.user_id = s.user_id
		JOIN td_user_type m ON s.type_id = m.type_id
		AND m.status = ?
		WHERE
			t.user_id = ?
	`

	var userTypes []UserWithType
	if err := m.db.SQL(query, models.STATUS_NORMAL, userID).Find(&userTypes); err != nil {
		return nil, err
	}
	return userTypes, nil
}

// CheckUserPhone 用户电话重复校验
func (m *UserDAO) CheckUserPhone(userid, phone, orgid string) (bool, error) {
	var users []model.SysUser
	dao := m.db.Where("phone=?", phone).And("org_id=?", orgid).And("status>?", models.STATUS_DEL)
	if userid != "" {
		dao.And("user_id <> '" + userid + "'")
	}
	err := dao.Find(&users)
	if err != nil {
		return false, err
	}
	if len(users) > 0 {
		return false, nil
	}
	return true, nil
}

// CheckUserName 用户账户名重复校验
func (m *UserDAO) CheckUserName(userid, username, orgid string) (bool, error) {
	var users []model.SysUser
	dao := m.db.Where("user_name=?", username).And("org_id=?", orgid).And("status>?", models.STATUS_DEL)
	if userid != "" {
		dao.And("user_id <> '" + userid + "'")
	}
	err := dao.Find(&users)
	if err != nil {
		return false, err
	}
	if len(users) > 0 {
		return false, nil
	}
	return true, nil
}

// GetForbidUsers 获取所有禁止人员信息
func (m *UserDAO) GetForbidUsers(caseid string) ([]model.TaForbidUser, error) {
	var forbiduser []model.TaForbidUser
	err := m.db.Where("case_id=?", caseid).And("(end_date is null or end_date > NOW())").And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&forbiduser)
	return forbiduser, err
}

// GetForbidUserByUserID 获取禁止人员信息
func (m *UserDAO) GetForbidUserByUserID(userid string) ([]model.TaForbidUser, error) {
	var forbiduser []model.TaForbidUser
	err := m.db.Where("user_id=?", userid).And("(end_date is null or end_date > NOW())").And("status>" + strconv.Itoa(models.STATUS_DEL)).Find(&forbiduser)
	return forbiduser, err
}

// SaveForbidUser 保存禁止人员信息
func (m *UserDAO) SaveForbidUser(forbid *model.TaForbidUser) error {
	forbid.BeginDate = time.Now().Local()
	forbid.Status = models.STATUS_NORMAL
	forbid.ForbidId = utils.GetGUID()
	_, err := m.db.Insert(forbid)
	return err
}

// OpenForbidUser 取消禁止信息
func (m *UserDAO) OpenForbidUser(forbid *model.TaForbidUser) error {
	forbid.EndDate = time.Now().Local()
	forbid.Status = models.STATUS_DEL
	var col = []string{
		"end_date",
		"status",
	}
	_, err := m.db.Cols(col...).Where("forbid_id=?", forbid.ForbidId).Update(forbid)
	return err
}

// UserWithCouponAndCard 用户关联卡券信息
type UserWithCouponAndCard struct {
	model.SysUser  `xorm:"extends"`
	CustomerCoupon []model.TaCustomerCoupon    `xorm:"-"`
	CustomerCard   []model.TaCustomerCard      `xorm:"-"`
	OrdersDetail   []model.TaGoodsOrdersDetail `xorm:"-"`
}

// GetCaseUserByType 获取案场所有销售信息
func (m *UserDAO) GetCaseUserByType(caseid, usertype string) ([]UserWithCouponAndCard, error) {
	var users []UserWithCouponAndCard
	sql := `select a.* from sys_user a inner join sys_user_type b on a.user_id=b.user_id
	inner join sys_user_case c on a.user_id=c.user_id
	where a.status>? and c.case_id =? and b.type_id=?`
	err := m.db.Sql(sql, models.STATUS_DEL, caseid, usertype).Find(&users)
	return users, err
}

// GetUserDetailByID 根据id获取案场销售
func (m *UserDAO) GetUserDetailByID(userid string) (*UserWithCouponAndCard, error) {
	var user UserWithCouponAndCard
	sql := `select * from sys_user where status>? and user_id=?`
	_, err := m.db.Sql(sql, models.STATUS_DEL, userid).Get(&user)
	return &user, err
}
