package vipcard

import (
	"math/rand"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/xorm"
)

// ChannelDAO 当前数据库操作对象
type VipcardDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewVipcardDAO New Inst
func NewVipcardDAO(ctx *utils.Context) *VipcardDAO {
	return &VipcardDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

type VipChild struct {
	model.TaVipCardChild `xorm:"extends"`
	CaseName             string
	CustomerName         string
}

// GetVipCardList获取VIP卡列表
func (m *VipcardDAO) GetVipCardList(caseids, cardNo, sellerName, userName string, page int, pageSize int) ([]VipChild, error) {
	var vipcards []VipChild
	sql := `SELECT
	vcc.*,
	sc.case_name,
	tc.customer_name
FROM
	ta_vip_card_child vcc
	INNER JOIN ta_vip_card vc ON vcc.vip_card_id = vc.vip_card_id
	INNER JOIN sys_case sc ON vc.case_id = sc.case_id 
	LEFT JOIN ta_customer tc on vcc.customer_id = tc.customer_id
WHERE
	vcc.status > ` + strconv.Itoa(models.STATUS_DEL) +
		` AND sc.case_id IN ('` + strings.Replace(caseids, ",", "','", -1) + `') `

	if cardNo != "" {
		sql = sql + ` and vcc.vip_card_child_code like '%` + cardNo + `%'`
	}

	if sellerName != "" {
		sql = sql + ` and tc.recommend_name like '%` + sellerName + `%'`
	}

	if userName != "" {
		sql = sql + ` and tc.customer_name like '%` + userName + `%'`
	}
	sql = sql + " order by vc.create_date desc limit " + strconv.Itoa((page-1)*pageSize) + ", " + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&vipcards)
	return vipcards, err
}

func (m *VipcardDAO) GetCardCount(caseids, cardNo, sellerName, userName string) (int, error) {
	var vipcards []VipChild
	sql := `SELECT
	vcc.*,
	sc.case_name,
	tc.customer_name
FROM
	ta_vip_card_child vcc
	INNER JOIN ta_vip_card vc ON vcc.vip_card_id = vc.vip_card_id
	INNER JOIN sys_case sc ON vc.case_id = sc.case_id 
	LEFT JOIN ta_customer tc on vcc.customer_id = tc.customer_id
WHERE
	vcc.status > ` + strconv.Itoa(models.STATUS_DEL) +
		` AND sc.case_id IN('` + strings.Replace(caseids, ",", "','", -1) + `') `
	if cardNo != "" {
		sql = sql + ` and vcc.vip_card_child_code like '%` + cardNo + `%'`
	}

	if sellerName != "" {
		sql = sql + ` and tc.recommend_name like '%` + sellerName + `%'`
	}

	if userName != "" {
		sql = sql + ` and tc.customer_name like '%` + userName + `%'`
	}
	err := m.db.Sql(sql).Find(&vipcards)
	return len(vipcards), err

}

// AddVipCard 新增VIP卡
func (m *VipcardDAO) AddVipCard(vipCard model.TaVipCard) (*model.TaVipCard, error) {
	vipCard.VipCardName = "面值" + vipCard.CardAmount + "元VIP卡"
	vipCard.VipCardId = utils.GetGUID()
	vipCard.CreateDate = time.Now()
	vipCard.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(vipCard)
	return &vipCard, err

}

// AddVipChildCard 新增VIP子卡
func (m *VipcardDAO) AddVipChildCard(vipChild model.TaVipCardChild) (*model.TaVipCardChild, error) {
	vipChild.CreateDate = time.Now()
	vipChild.VipCardChildId = utils.GetGUID()
	vipChild.Status = models.STATUS_READY
	_, err := m.db.Insert(vipChild)
	return &vipChild, err
}

// GetVipChildByCode 通过code获取VIP子卡
func (m *VipcardDAO) GetVipChildByCode(vipCode string) (*model.TaVipCardChild, error) {
	var vipChild []model.TaVipCardChild
	err := m.db.Where("status>"+strconv.Itoa(models.STATUS_DEL)).And("vip_card_child_code =?", vipCode).Find(&vipChild)
	if err != nil {
		return nil, err
	}
	if len(vipChild) > 0 {
		return &vipChild[0], nil
	}
	return nil, nil
}

// GetVipCardById 根据id获取VIP卡
func (m *VipcardDAO) GetVipCardById(id string) (*model.TaVipCard, error) {
	var vipCard []model.TaVipCard
	err := m.db.Where("status>"+strconv.Itoa(models.STATUS_DEL)).And("vip_card_id = ?", id).Find(&vipCard)
	if err != nil {
		return nil, err
	}
	if len(vipCard) > 0 {
		return &vipCard[0], nil
	}
	return nil, nil
}

// UpdateVipChild 更新子卡
func (m *VipcardDAO) UpdateVipChild(vipChild *model.TaVipCardChild) error {
	vipChild.ActiveDate = time.Now()
	var cols = []string{
		"customer_id",
		"customer_tel",
		"active_date",
		"status",
		"sales_id",
		"sales_name",
	}
	_, err := m.db.Cols(cols...).Where("vip_card_child_id = ?", vipChild.VipCardChildId).Update(vipChild)
	return err
}

// GenerateChildCode 生成子卡code
func (m *VipcardDAO) GenerateChildCode() string {
	var temp1 int = rand.Intn(9)*87 + 11
	var temp2 int = rand.Intn(9)*89 + 13
	nano := time.Now().UnixNano()
	var nanostr string = strconv.FormatInt(nano, 10)
	var temp3 string = nanostr[11:]
	var temp4 string = "8"
	var temp5 int = rand.Intn(9)*7 + 17
	var code string = "88" + strconv.Itoa(temp1) + temp4 + strconv.Itoa(temp2) + temp3 + strconv.Itoa(temp5)
	return code
}

// GetVipCardListExcel 获取VIP卡列表 excel
func (m *VipcardDAO) GetVipCardListExcel(caseids, cardNo, sellerName, userName string) ([]VipChild, error) {
	var vipcards []VipChild
	sql := `SELECT
	vcc.*,
	sc.case_name,
	tc.customer_name
FROM
	ta_vip_card_child vcc
	INNER JOIN ta_vip_card vc ON vcc.vip_card_id = vc.vip_card_id
	INNER JOIN sys_case sc ON vc.case_id = sc.case_id 
	LEFT JOIN ta_customer tc on vcc.customer_id = tc.customer_id
WHERE
	vcc.status > ` + strconv.Itoa(models.STATUS_DEL) +
		` AND sc.case_id IN ('` + strings.Replace(caseids, ",", "','", -1) + `') `

	if cardNo != "" {
		sql = sql + ` and vcc.vip_card_child_code like '%` + cardNo + `%'`
	}

	if sellerName != "" {
		sql = sql + ` and tc.recommend_name like '%` + sellerName + `%'`
	}

	if userName != "" {
		sql = sql + ` and tc.customer_name like '%` + userName + `%'`
	}
	sql = sql + " order by vc.create_date, vip_card_child_code desc"
	err := m.db.Sql(sql).Find(&vipcards)
	return vipcards, err
}

// AddCustomerVip 保存客户Vip卡信息
func (m *VipcardDAO) AddCustomerVip(customerVip model.TaCustomerVip) (*model.TaCustomerVip, error) {
	customerVip.CreateDate = time.Now()
	customerVip.CustomerVipId = utils.GetGUID()
	customerVip.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(customerVip)
	return &customerVip, err
}

// InsertCustomerVipChange 保存客户Vip卡变动
func (m *VipcardDAO) InsertCustomerVipChange(change model.TaCustomerVipChange) error {
	change.ChangeId = utils.GetGUID()
	change.Status = models.STATUS_NORMAL
	change.CreateDate = time.Now()

	_, err := m.db.Insert(change)
	return err
}

// CustomerVipWithChange 用户vip卡关联变更
type CustomerVipWithChange struct {
	model.TaCustomerVip `xorm:"extends"`
	Changes             []model.TaCustomerVipChange
}

// GetCustomerVips 获取客户vip卡信息
func (m *VipcardDAO) GetCustomerVips(customerid string) ([]CustomerVipWithChange, error) {
	var vips []CustomerVipWithChange
	sql := `select * from ta_customer_vip where customer_id=? and status>?`
	err := m.db.Sql(sql, customerid, models.STATUS_DEL).Find(&vips)
	if err != nil {
		return nil, err
	}
	for inx, vip := range vips {
		var changes []model.TaCustomerVipChange
		m.db.Where("customer_vip_id=?", vip.CustomerVipId).And("status>?", models.STATUS_DEL).Find(&changes)
		vips[inx].Changes = changes
	}
	return vips, nil
}
