package routers

import (
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/controllers/bodycheck"
	"spaceofcheng/services/controllers/cases"
	"spaceofcheng/services/controllers/course"
	"spaceofcheng/services/controllers/customer"
	"spaceofcheng/services/controllers/customerremark"
	"spaceofcheng/services/controllers/goods"
	"spaceofcheng/services/controllers/message"
	"spaceofcheng/services/controllers/user"

	"github.com/astaxie/beego"
)

func getGuestRoutes(prefix string) beego.LinkNamespace {
	return beego.NSNamespace(prefix,
		// cms
		beego.NSRouter("/cms/info", &message.MessageController{}, "get:GetCmsInfoByLocation"),
		beego.NSRouter("/cms/img", &message.MessageController{}, "get:GetImgByLocation"),
		beego.NSRouter("/cms/news", &message.MessageController{}, "get:GetNewsByLocation"),
		beego.NSRouter("/cms/case", &message.MessageController{}, "get:GetCmsCaseList"),
		beego.NSRouter("/cms/location", &message.MessageController{}, "get:GetLocations"),
		beego.NSRouter("/cms/location/detail", &message.MessageController{}, "get:GetLocationInfoList"),
		beego.NSRouter("/cms/course", &course.CourseController{}, "get:GetCourseByLocation"),
		beego.NSRouter("/cms/case/:cmscaseid", &message.MessageController{}, "get:GetCmsCaseByID"),

		// 课程
		beego.NSRouter("/course/:courseid", &course.CourseController{}, "get:GetCourseByID"),

		// 案场
		beego.NSRouter("/case", &cases.CaseController{}, "get:GetOrgCases"),
		beego.NSRouter("/cms/area", &cases.CaseController{}, "get:GetAreaByCase"),

		// 商品制作状态
		beego.NSRouter("/dict/order/makestatus", &goods.GoodsController{}, "get:GetMakeStatus"),

		// 商品
		beego.NSRouter("/goods", &goods.GoodsController{}, "get:GetListForWechat"),
		beego.NSRouter("/type/goods", &goods.GoodsController{}, "get:GetGoodsTypeByCase"),

		// 验证码 获取, 校验
		beego.NSRouter("/captcha", &controllers.BaseController{}, "get:SendCaptcha"),
		beego.NSRouter("/captcha", &controllers.BaseController{}, "post:ValidCaptcha"),

		// 微信 登录, 注册
		beego.NSRouter("/wxsignin", &customer.CustomerController{}, "post:SignIn"),
		beego.NSRouter("/wxsignup", &customer.CustomerController{}, "post:SignUp"),

		// 登录
		beego.NSRouter("/signin", &user.UserController{}, "post:SignIn"),

		// 微信
		// js sdk 签名
		beego.NSRouter("/wechat/jssdk/signature", &controllers.WechatController{}, "get:GetWxJsSDKSignature"),
		beego.NSRouter("/wechat/mini/login", &controllers.WechatController{}, "post:MiniLogin"),
		beego.NSRouter("/wechat/mini/decodedata", &controllers.WechatController{}, "post:DecodeMiniData"),

		beego.NSRouter("/websocket/:grps/:id", &controllers.BaseController{}, "get:Ws"),

		// 体检参数接收
		beego.NSRouter("/PostCheckResult", &bodycheck.BodyCheckController{}, "post:PostCheckResult"),
		beego.NSRouter("/GetQrcodeURL", &bodycheck.BodyCheckController{}, "get:GetQrcodeURL"),

		// 签到
		beego.NSRouter("/case/checkin", &cases.SigninController{}, "post:AddSignin"),
		beego.NSRouter("/wechat", &controllers.WechatController{}, "get,post:WechatInfo"),
		// beego.NSRouter("/wechat", &controllers.WechatController{}, "post:WxReceive"),

		beego.NSRouter("/customerremark/:recommendCode", &customerremark.CustomerRemarkController{}, "get:IsExist"),
	)
}
