package bodychecklist

import (
	"errors"
	"spaceofcheng/services/models/bodychecklist"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strings"
)

// BodychecklistServ 系统处理
type BodychecklistServ struct {
	ctx *utils.Context
	dao *bodychecklist.BodychecklistDAO
}

// NewBodychecklistServ 初始化
func NewBodychecklistServ(ctx *utils.Context) *BodychecklistServ {
	return &BodychecklistServ{
		ctx: ctx,
		dao: bodychecklist.NewBodychecklistDAO(ctx),
	}
}

func (s *BodychecklistServ) GetBodyCheckList(caseID, phone, customerName string, pagenavi ...int) ([]bodychecklist.BodyCheck, int64, error) {
	filters := ""
	if caseID != "" {
		if err := utils.NewAuthEngine(s.ctx).CheckCase(caseID); err != nil {
			return nil, 0, err
		}

		filters = "a.case_id='" + caseID + "'"
	} else {
		casesRaw := s.ctx.Get("cases")
		if casesRaw == nil {
			return nil, 0, errors.New("请设置过滤案场")
		}

		cases := casesRaw.([]model.SysUserCase)
		caseIDs := []string{}
		for _, cs := range cases {
			caseIDs = append(caseIDs, cs.CaseId)
		}

		filters = "a.case_id in ('" + strings.Join(caseIDs, "','") + "')"
	}

	limit := utils.GetPageNaviLimit(pagenavi...)
	res, total, err := s.dao.GetBodychecklist(phone, customerName, filters, limit)
	if err != nil {
		utils.LogError("查询体检列表失败: " + err.Error())
		return nil, 0, errors.New("查询体检列表失败")
	}
	return res, total, nil
}
