package cases

import (
	"errors"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// GetCaseEquipment 获取体检设备
func (s *CaseServ) GetCaseEquipment(caseids string, page int, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	specs, err := s.dao.GetEquipmentList(caseids, page, pageSize)
	if err != nil {
		utils.LogError("获取体检设备信息失败: " + err.Error())
		return nil, errors.New("获取体检设备信息失败")
	}
	total, err := s.dao.GetEquipmentCount(caseids)
	if err != nil {
		utils.LogError("获取体检设备信息失败: " + err.Error())
		return nil, errors.New("获取体检设备信息失败")
	}

	return map[string]interface{}{
		"list":     specs,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// GetCaseEquipmentByID 根据ID获取体检设备
func (s *CaseServ) GetCaseEquipmentByID(equipmentid string) (*model.SysCaseEquipment, error) {
	equipment, err := s.dao.GetCaseEquipmentByID(equipmentid)
	if err != nil {
		utils.LogError("获取体检设备信息失败: " + err.Error())
		return nil, errors.New("获取体检设备信息失败")
	}
	return equipment, nil
}

// SaveCaseEquipment 保存体检设备
func (s *CaseServ) SaveCaseEquipment(equipment model.SysCaseEquipment) (*model.SysCaseEquipment, error) {
	var err error
	if equipment.Name == "" {
		return nil, utils.LogError("体检设备名称不能为空！")
	}
	if equipment.EquipmentId == "" {
		_, err = s.dao.AddCaseEquipment(&equipment)
	} else {
		err = s.dao.UpdateCaseEquipment(equipment)
	}
	if err != nil {
		utils.LogError("保存体检设备信息失败: " + err.Error())
		return nil, errors.New("保存体检设备信息失败")
	}
	return &equipment, nil
}

// DelCaseEquipment 删除体检设备
func (s *CaseServ) DelCaseEquipment(equipmentid string) error {
	err := s.dao.DelCaseEquipment(equipmentid)
	if err != nil {
		utils.LogError("删除体检设备信息失败: " + err.Error())
		return errors.New("删除体检设备信息失败")
	}
	return nil
}
