package cases

import (
	"errors"
	"spaceofcheng/services/models/cases"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// GetCaseRecord 获取案场月记录列表
func (s *CaseServ) GetCaseRecord(caseids string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	records, err := s.dao.GetCaseRecord(caseids, page, pageSize)
	if err != nil {
		utils.LogError("获取案场月记录失败: " + err.Error())
		return nil, errors.New("获取案场月记录失败")
	}
	total, err := s.dao.GetCaseRecordCount(caseids)
	if err != nil {
		utils.LogError("获取案场月记录失败: " + err.Error())
		return nil, errors.New("获取案场月记录失败")
	}

	return map[string]interface{}{
		"list":     records,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// AddRecord 保存案场月记录信息
func (s *CaseServ) AddRecord(record model.SysCaseRecord) (*model.SysCaseRecord, error) {
	res, err := s.dao.GetRecordByMonth(record.CaseId, record.Month)
	if err != nil {
		utils.LogError("保存月记录失败: " + err.Error())
		return nil, errors.New("保存月记录失败")
	}
	if res != nil {
		return nil, errors.New("案场已存在选择月份的记录！不允许新增！")
	}
	newinfo, err := s.dao.AddRecord(record)
	if err != nil {
		utils.LogError("保存月记录失败: " + err.Error())
		return nil, errors.New("保存月记录失败")
	}
	return newinfo, nil
}

// GetRecordByID 获取月记录信息
func (s *CaseServ) GetRecordByID(recordid string) (*cases.CaseRecord, error) {
	record, err := s.dao.GetRecordByID(recordid)
	if err != nil {
		utils.LogError("获取月记录失败: " + err.Error())
		return nil, errors.New("获取月记录失败")
	}
	return record, nil
}
