package cases

import (
	"errors"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// GetCaseTable 获取案场桌位
func (s *CaseServ) GetCaseTable(caseids string, page int, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	areas, err := s.dao.GetCaseTable(caseids, page, pageSize)
	if err != nil {
		utils.LogError("获取案场桌位信息失败: " + err.Error())
		return nil, errors.New("获取案场桌位信息失败")
	}
	total, err := s.dao.GetCaseTableCount(caseids)
	if err != nil {
		utils.LogError("获取案场桌位信息失败: " + err.Error())
		return nil, errors.New("获取案场桌位信息失败")
	}

	return map[string]interface{}{
		"list":     areas,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// GetCaseTableByID 根据桌位ID获取桌位
func (s *CaseServ) GetCaseTableByID(tableid string) (*model.SysCaseTable, error) {
	areas, err := s.dao.GetCaseTableByID(tableid)
	if err != nil {
		utils.LogError("获取案场桌位信息失败: " + err.Error())
		return nil, errors.New("获取案场桌位信息失败")
	}
	return areas, nil
}

// SaveCaseTable 保存桌位
func (s *CaseServ) SaveCaseTable(table model.SysCaseTable) (*model.SysCaseTable, error) {
	var err error
	if table.TableNo == "" {
		return nil, utils.LogError("桌位名称不能为空")
	}
	if table.TableId == "" {
		_, err = s.dao.AddCaseTable(&table)
	} else {
		err = s.dao.UpdateCaseTable(table)
	}
	if err != nil {
		utils.LogError("保存案场桌位信息失败: " + err.Error())
		return nil, errors.New("保存案场桌位信息失败")
	}
	return &table, nil
}

// DelCaseTable 删除桌位
func (s *CaseServ) DelCaseTable(tableid string) error {
	err := s.dao.DelCaseTable(tableid)
	if err != nil {
		utils.LogError("删除案场桌位信息失败: " + err.Error())
		return errors.New("删除案场桌位信息失败")
	}
	return nil
}
