package customerremark

import (
	"errors"
	"spaceofcheng/services/models/customerremark"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

type CustomerRemarkServ struct {
	ctx *utils.Context
	dao *customerremark.CustomerRemarkDAO
}

// NewCustomerRemarkServ 初始化
func NewCustomerRemarkServ(ctx *utils.Context) *CustomerRemarkServ {
	return &CustomerRemarkServ{
		ctx: ctx,
		dao: customerremark.NewCustomerRemarkDAO(ctx),
	}
}

func (s *CustomerRemarkServ) GetCustomerRemarkList(salesId, customerId string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	remarks, err := s.dao.GetRemarkList(salesId, customerId, page, pageSize)
	if err != nil {
		utils.LogError("获取客户备注列表失败: " + err.Error())
		return nil, errors.New("获取客户备注列表失败")
	}
	total, err := s.dao.GetRemarkListCount(salesId, customerId)
	if err != nil {
		utils.LogError("获取客户备注列表失败: " + err.Error())
		return nil, errors.New("获取客户备注列表失败")
	}
	return map[string]interface{}{
		"list":     remarks,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

func (s *CustomerRemarkServ) GetCustomerReceiveRecord(customerId, salesId string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	record, err := s.dao.CustomerReceiveRecord(customerId, salesId, page, pageSize)
	if err != nil {
		utils.LogError("获取客户领取列表失败: " + err.Error())
		return nil, errors.New("获取客户领取列表失败")
	}
	total, err := s.dao.CustomerReceiveRecordCount(customerId, salesId)
	if err != nil {
		utils.LogError("获取客户领取列表失败: " + err.Error())
		return nil, errors.New("获取客户领取列表失败")
	}
	return map[string]interface{}{
		"list":     record,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}
func (s *CustomerRemarkServ) AddRemark(remark model.TaSaleCustomerRemark) (*model.TaSaleCustomerRemark, error) {
	newRemark, err := s.dao.AddRemark(remark)
	if err != nil {
		utils.LogError("保存客户备注失败: " + err.Error())
		return nil, errors.New("保存客户备注失败")
	}
	return newRemark, nil
}
func (s *CustomerRemarkServ) SearchCustomer(customerInfo, salesId string) (*model.TaCustomer, error) {
	customer, err := s.dao.SearchCustomer(customerInfo, salesId)
	if err != nil {
		utils.LogError("获取客户信息失败: " + err.Error())
		return nil, errors.New("获取客户信息失败")
	}
	return &customer, nil
}
func (s *CustomerRemarkServ) IsExist(recommendCode string) (int, error) {
	num, err := s.dao.IsExist(recommendCode)
	if err != nil {
		utils.LogError("获取推荐人失败: " + err.Error())
		return 0, errors.New("获取推荐人失败")
	}
	return num, nil
}
