package flashbuy

import (
	"errors"
	"fmt"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/course"
	"spaceofcheng/services/models/flashbuy"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/verify"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"time"

	"github.com/astaxie/beego"
)

// FlashBuyServ 系统处理
type FlashBuyServ struct {
	ctx       *utils.Context
	dao       *flashbuy.FlashbuyDAO
	vdao      *verify.VerifyDAO
	courseDAO *course.CourseDAO
}

// NewFlashBuyServ 初始化
func NewFlashBuyServ(ctx *utils.Context) *FlashBuyServ {
	return &FlashBuyServ{
		ctx:       ctx,
		dao:       flashbuy.NewFlashbuyDAO(ctx),
		vdao:      verify.NewVerifyDAO(ctx),
		courseDAO: course.NewCourseDAO(ctx),
	}
}

func (s *FlashBuyServ) GetFlashBuyList(caseid, flashBuyName, flashBuyStatus string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	flashbuys, err := s.dao.GetFlashBuyList(caseid, flashBuyName, flashBuyStatus, page, pageSize)
	if err != nil {
		utils.LogError("获取抢购活动列表失败: " + err.Error())
		return nil, errors.New("获取抢购活动列表失败")
	}
	total, err := s.dao.GetFlashBuyListCount(caseid, flashBuyName, flashBuyStatus)
	if err != nil {
		utils.LogError("获取抢购活动列表失败: " + err.Error())
		return nil, errors.New("获取抢购活动列表失败")
	}
	return map[string]interface{}{
		"list":     flashbuys,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}
func (s *FlashBuyServ) GetFlashBuyById(flashBuyId string) (*model.TaFlashBuy, error) {
	flashbuy, err := s.dao.GetFlashBuyById(flashBuyId)
	if err != nil {
		utils.LogError("获取抢购活动信息失败: " + err.Error())
		return nil, errors.New("获取抢购活动信息失败")
	}
	return flashbuy, nil
}

// GetFlashByUser 获取用户的抢购信息
func (s *FlashBuyServ) GetFlashByUser(id string) ([]model.TaCustomerFlashBuy, error) {
	cst := s.ctx.Get("customer")
	if cst == nil {
		return nil, errors.New("系统内部错误")
	}
	flashbuys, err := s.dao.GetCustomerFlashBuy(id, cst.(model.TaCustomer).CustomerId)
	return flashbuys, err
}

func (s *FlashBuyServ) SaveFlashBuy(flashbuy model.TaFlashBuy) (*model.TaFlashBuy, error) {
	var newFlashBuy *model.TaFlashBuy
	var err error
	if flashbuy.CaseId == "" {
		return nil, utils.LogError("请选择所属案场！")
	}
	if flashbuy.FlashBuyName == "" {
		return nil, utils.LogError("请输入活动标题！")
	}
	if flashbuy.FlashBuyInfo == "" {
		return nil, utils.LogError("请输入活动描述！")
	}
	if flashbuy.ModelId == "" {
		return nil, utils.LogError("请选择活动模板！")
	}
	if flashbuy.FlashBuyId == "" {
		org := s.ctx.Get("org").(model.SysOrg)
		flashbuy.OrgId = org.OrgId
		newFlashBuy, err = s.dao.AddNewFlashBuy(flashbuy)
	} else {
		err = s.dao.EditFlashBuy(flashbuy)
		newFlashBuy = &flashbuy
	}
	if err != nil {
		utils.LogError("保存抢购活动信息失败: " + err.Error())
		return nil, errors.New("保存抢购活动信息失败")
	}
	return newFlashBuy, nil
}

func (s *FlashBuyServ) UpdateFlashBuy(flashBuyId, flashBuyStatus string) error {
	err := s.dao.UpdateFlashBuy(flashBuyId, flashBuyStatus)
	if err != nil {
		utils.LogError("更新活动状态失败: " + err.Error())
		return errors.New("更新活动状态失败")
	}
	return err
}
func (s *FlashBuyServ) DeleteFlashBuy(flashBuyId string) error {
	err := s.dao.DeleteFlashBuy(flashBuyId)
	if err != nil {
		utils.LogError("删除抢购活动信息失败: " + err.Error())
		return errors.New("删除抢购活动信息失败")
	}
	return err
}

func (s *FlashBuyServ) GetCustomerFlashBuyById(flashBuyId, phone string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	flashbuys, err := s.dao.GetCustomerFlashBuyById(flashBuyId, phone, page, pageSize)
	if err != nil {
		utils.LogError("获取记录列表失败: " + err.Error())
		return nil, errors.New("获取获取记录列表失败")
	}
	total, err := s.dao.GetCustomerFlashBuyByIdCount(flashBuyId, phone)
	if err != nil {
		utils.LogError("获取记录列表失败: " + err.Error())
		return nil, errors.New("获取获取记录列表失败")
	}
	return map[string]interface{}{
		"list":     flashbuys,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

func (s *FlashBuyServ) VerifyCustomerFlashBuyList(qrcode, caseId string) (*flashbuy.CustomerFlashBuy, error) {
	var err error
	var customerFlashBuyId string
	var customerVerify *flashbuy.CustomerFlashBuy
	customerFlashBuyId, err = s.vdao.GetCustomerCourseIdByQrcode(qrcode)
	beego.Error("________________________________")
	beego.Error(customerFlashBuyId)
	beego.Error("________________________________")
	if err != nil {
		utils.LogError("获取核销列表失败: " + err.Error())
		return nil, errors.New("获取核销列表失败")
	}
	customerVerify, err = s.dao.GetCustomerFlashBuyByQr(customerFlashBuyId, caseId)
	if err != nil {
		utils.LogError("获取核销列表失败: " + err.Error())
		return nil, errors.New("获取核销列表失败")
	}
	return customerVerify, nil

}

func (s *FlashBuyServ) Verify(customerFlashBuyId string) error {
	err := s.dao.VerifyFlashBuy(customerFlashBuyId)
	if err != nil {
		utils.LogError("核销失败: " + err.Error())
		return errors.New("核销失败")
	}
	return nil
}

func (s *FlashBuyServ) GetCustomerFlashBuyByCustomerId(customerId string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	flashbuys, err := s.dao.GetCustomerFlashBuyByCustomerId(customerId, page, pageSize)
	if err != nil {
		utils.LogError("获取抢购活动列表失败: " + err.Error())
		return nil, errors.New("获取抢购活动列表失败")
	}
	total, err := s.dao.GetCustomerFlashBuyByCustomerIdCount(customerId)
	if err != nil {
		utils.LogError("获取抢购活动列表失败: " + err.Error())
		return nil, errors.New("获取抢购活动列表失败")
	}
	return map[string]interface{}{
		"list":     flashbuys,
		"pageSize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

func (s *FlashBuyServ) GetCustomerFlashBuyId(customerFlashBuyId string) (*flashbuy.CustomerFlashDetail, error) {
	flashbuy, err := s.dao.GetCustomerFlashBuyId(customerFlashBuyId)
	if err != nil {
		utils.LogError("获取抢购活动信息失败: " + err.Error())
		return nil, errors.New("获取抢购活动信息失败")
	}
	return flashbuy, nil
}

// FlashBuy 抢购
func (s *FlashBuyServ) FlashBuy(id string) error {
	flashbuy, err := s.dao.GetFlashBuyById(id)
	if err != nil {
		utils.LogError("获取抢购活动信息失败: " + err.Error())
		return errors.New("获取抢购活动信息失败！")
	}
	if flashbuy.Status != models.STATUS_NORMAL {
		return errors.New("抢购活动状态异常！")
	}
	if time.Now().After(flashbuy.EndDate) {
		return errors.New("活动已结束！")
	}
	if flashbuy.FlashBuyStatus != models.FLASH_BUY_ACTIVE {
		return errors.New("活动已停用！")
	}
	if time.Now().Before(flashbuy.StartDate) {
		return errors.New("活动还未开始！")
	}
	customer := s.ctx.Get("customer").(model.TaCustomer)
	buys, err := s.dao.GetCustomerFlashBuy(id, customer.CustomerId)
	if err != nil {
		utils.LogError("获取客户抢购信息失败: " + err.Error())
		return errors.New("获取客户抢购信息失败！")
	}
	if len(buys) > 0 {
		return errors.New("您已参与过该活动！")
	}
	if flashbuy.JoinNum >= flashbuy.FlashBuyMaxAttendant {
		err = s.dao.UpdateFlashBuy(flashbuy.FlashBuyId, models.FLASH_BUY_OVER)
		if err != nil {
			utils.LogError("更新状态失败: " + err.Error())
		}
		return errors.New("活动已结束！")
	}
	var customerFlashBuy model.TaCustomerFlashBuy
	customerFlashBuy.FlashBuyId = flashbuy.FlashBuyId
	customerFlashBuy.CustomerId = customer.CustomerId
	customerFlashBuy.CaseId = flashbuy.CaseId
	customerFlashBuy.OrgId = flashbuy.OrgId
	if flashbuy.ValidateType == models.FLASH_VALIDATE_DATE {
		customerFlashBuy.ValidateStart = flashbuy.StartDate
		customerFlashBuy.ValidateEnd = flashbuy.EndDate
	} else {
		customerFlashBuy.ValidateStart = time.Now().Local()
		customerFlashBuy.ValidateEnd = time.Now().Local().AddDate(0, 0, flashbuy.ValidateDays)
	}
	recored, err := s.dao.SaveCustomerFlashBuy(customerFlashBuy)
	if err != nil {
		utils.LogError("保存用户抢购信息失败: " + err.Error())
		return errors.New("抢购失败！")
	}
	num, err := s.dao.UpdateFlashBuyJoin(flashbuy.FlashBuyId)
	if err != nil {
		utils.LogError("抢购失败: " + err.Error())
		return errors.New("抢购失败！")
	}
	if num < 1 {
		err = s.dao.UpdateFlashBuy(flashbuy.FlashBuyId, models.FLASH_BUY_OVER)
		if err != nil {
			utils.LogError("更新状态失败: " + err.Error())
		}
		return errors.New("活动已结束！")
	}
	custQRCode := model.TaCustomerCourseQrcode{
		CustomerCourseId: recored.CustomerFlashBuyId,
		CustomerQrcode:   utils.GenerateQRCode(),
		CreateDate:       time.Now().Local(),
		Status:           models.STATUS_NORMAL,
		QrcodeType:       models.QRCODE_TYPE_FLASH,
	}

	if err := s.courseDAO.SaveCustomerCourseQrcode(&custQRCode); err != nil {
		utils.LogError("插入中奖二维码失败: " + err.Error())
		return errors.New("写入中奖二维码失败")
	}

	// 推送微信消息
	userMap := s.ctx.Get("userMap").(model.TaUserMapping)
	userCases := s.ctx.Get("cases").([]model.SysUserCase)
	clienturl := s.ctx.Get("clienturl").(string)
	tplID := s.ctx.Get("tplid").(string)

	var caseName string
	if userCases != nil && len(userCases) > 0 {
		for _, cs := range userCases {
			if cs.CaseId == recored.CaseId {
				caseName = cs.CaseName
				break
			}
		}
	}

	remarkTpl := "请在%s日前, 前往%s兑换"

	message := utils.Message{
		To: utils.ClientID{ID: userMap.Openid},
		Data: map[string]interface{}{
			"orgid": recored.OrgId,
			"tplid": tplID,
			"link":  clienturl,
			"data": map[string]string{
				"first":    "抢券成功提醒",
				"keyword1": flashbuy.FlashBuyName,
				"keyword2": time.Now().Local().Format("2006-01-02"),
				"remark":   fmt.Sprintf(remarkTpl, recored.ValidateEnd.Format("2006-01-02"), caseName),
			},
		},
	}
	go utils.SendWechat(message)

	return nil
}

func (s *FlashBuyServ) GetFlashModelList() ([]model.TdFlashbuyModel, error) {
	model, err := s.dao.GetFlashModelList()
	if err != nil {
		utils.LogError("获取抢购活动模板失败: " + err.Error())
		return nil, errors.New("获取抢购活动模板失败")
	}
	return model, nil
}

func (s *FlashBuyServ) AddNewFlashBuyCustomer(customer model.TaCustomer, flashBuyId string) error {
	var flashBuyCustomer = model.TaFlashBuyCustomer{
		FlashBuyId: flashBuyId,
		CustomerId: customer.CustomerId,
	}
	if customer.Phone == "" {
		flashBuyCustomer.IsNew = 1
	} else {
		flashBuyCustomer.IsNew = 0
	}
	err := s.dao.AddNewFlashBuyCustomer(flashBuyCustomer)
	if err != nil {
		utils.LogError("存储用户抢购记录失败: " + err.Error())
		return errors.New("存储用户抢购记录失败")
	}
	return nil
}
func (s *FlashBuyServ) UpdateFlashBuyCustomer(customerId, flashBuyId string) error {
	err := s.dao.UpdateFlashBuyCustomer(customerId, flashBuyId)
	if err != nil {
		utils.LogError("更新用户抢购记录失败: " + err.Error())
		return errors.New("更新用户抢购记录失败")
	}
	return nil
}

func (s *FlashBuyServ) IsFlashBuyCustomer(customerId, flashBuyId string) (bool, error) {
	flashBuyCustomer, err := s.dao.IsFlashBuyCustomer(customerId, flashBuyId)
	if err != nil {
		utils.LogError("判断是否参与失败: " + err.Error())
		return false, errors.New("判断是否参与失败")
	}
	if flashBuyCustomer != nil || flashBuyCustomer.IsNew == 0 {
		return false, errors.New("客户已参与本次抢购")
	}
	return true, nil
}
