package goods

import (
	"errors"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// GetGoodsSpec 获取商品规格
func (s *GoodsServ) GetGoodsSpec(caseids string, page int, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	specs, err := s.dao.GetGoodsSpec(caseids, page, pageSize)
	if err != nil {
		utils.LogError("获取商品规格失败: " + err.Error())
		return nil, errors.New("获取商品规格失败")
	}
	total, err := s.dao.GetGoodsSpecCount(caseids)
	if err != nil {
		utils.LogError("获取商品规格失败: " + err.Error())
		return nil, errors.New("获取商品规格失败")
	}

	return map[string]interface{}{
		"list":     specs,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// GetGoodsSpecByID 根据商品类型ID获取商品类型
func (s *GoodsServ) GetGoodsSpecByID(specid string) (*model.TdSpec, error) {
	goodsSpec, err := s.dao.GetGoodsSpecByID(specid)
	if err != nil {
		utils.LogError("获取商品规格失败: " + err.Error())
		return nil, errors.New("获取商品规格失败")
	}
	return goodsSpec, nil
}

// SaveGoodsSpec 保存商品类型
func (s *GoodsServ) SaveGoodsSpec(goodsSpec model.TdSpec) (*model.TdSpec, error) {
	if goodsSpec.SpecName == "" {
		return nil, errors.New("规格名称不能为空！")
	}

	if goodsSpec.CaseId == "" {
		return nil, errors.New("案场不能为空！")
	}
	var err error
	if goodsSpec.SpecId == "" {
		_, err = s.dao.AddGoodsSpec(&goodsSpec)
	} else {
		err = s.dao.UpdateGoodsSpec(goodsSpec)
	}
	if err != nil {
		utils.LogError("保存商品规格失败: " + err.Error())
		return nil, errors.New("保存商品规格失败")
	}
	return &goodsSpec, nil
}

// DelGoodsSpec 删除商品类型
func (s *GoodsServ) DelGoodsSpec(specid string) error {
	goods, err := s.dao.GetGoodsBySpec(specid)
	if err != nil {
		utils.LogError("删除商品规格失败: " + err.Error())
		return errors.New("删除商品规格失败")
	}
	if len(goods) > 0 {
		return errors.New("已关联商品。不允许删除")
	}
	err = s.dao.DelGoodsSpec(specid)
	if err != nil {
		utils.LogError("删除商品规格失败: " + err.Error())
		return errors.New("删除商品规格失败")
	}
	return nil
}
