package message

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/course"
	"spaceofcheng/services/models/message"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"strings"
)

// GetCmsCasesList 获取项目列表
func (s *MessageServ) GetCmsCasesList(name, caseids string, page int, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	user := s.ctx.Get("user").(model.SysUser)
	infos, err := s.dao.GetCmsCaseList(name, caseids, user.UserId, page, pageSize)
	if err != nil {
		utils.LogError("获取项目列表失败: " + err.Error())
		return nil, errors.New("获取项目列表失败")
	}
	total, err := s.dao.GetCmsCaseListCount(name, caseids, user.UserId)
	if err != nil {
		utils.LogError("获取项目列表失败: " + err.Error())
		return nil, errors.New("获取项目列表失败")
	}

	return map[string]interface{}{
		"list":     infos,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// GetCmsCaseByOrg 获取项目列表
func (s *MessageServ) GetCmsCaseByOrg(orgid string) ([]message.CmsCase, error) {
	list, err := s.dao.GetCmsCaseByOrg(orgid)
	if err != nil {
		utils.LogError("获取项目列表失败: " + err.Error())
		return nil, errors.New("获取项目列表失败")
	}
	for i, c := range list {
		imgs, err := s.dao.GetCmsCaseImgs(c.CmsCaseId)
		if err != nil {
			utils.LogError("获取项目列表失败: " + err.Error())
			return nil, errors.New("获取项目列表失败")
		}
		list[i].CmsCaseImgs = imgs
	}
	return list, err
}

// GetCmsCasesByID 获取項目信息
func (s *MessageServ) GetCmsCasesByID(cmscaseid string) (*message.CmsCase, error) {
	var info = &message.CmsCase{}
	var err error
	info, err = s.dao.GetCmsCaseByID(cmscaseid)
	if err != nil {
		utils.LogError("获取项目信息失败: " + err.Error())
		return nil, errors.New("获取项目信息失败")
	}
	if info != nil {
		var courses []course.CourseDetail
		if info.IsAllCourse == 1 {
			if info.CaseId == "all" {
				courses, err = s.dao.GetCourseByOrgID(info.OrgId)
			} else {
				courses, err = s.dao.GetCourseByCaseID(info.CaseId)
			}
		} else {
			courses, err = s.dao.GetCoursesByCmsCase(info.CmsCaseId)
		}
		if err != nil {
			utils.LogError("获取项目信息失败: " + err.Error())
			return nil, errors.New("获取项目信息失败")
		}

		for inx, course := range courses {
			tags, err := s.coursedao.GetCourseTag(course.CourseId)
			if err != nil {
				utils.LogError("获取项目信息失败: " + err.Error())
				return nil, errors.New("获取项目信息失败")
			}
			courses[inx].CourseTags = tags
			details, err := s.coursedao.GetCourseDetail(course.CourseId)
			if err != nil {
				utils.LogError("获取项目信息失败: " + err.Error())
				return nil, errors.New("获取项目信息失败")
			}
			courses[inx].CourseDetail = details
			imgs, err := s.coursedao.GetCourseImgs(course.CourseId)
			if err != nil {
				utils.LogError("获取项目信息失败: " + err.Error())
				return nil, errors.New("获取项目信息失败")
			}
			courses[inx].CourseImgs = imgs
			caseinfo, err := s.casedao.GetCaseByID(course.CaseId)
			if err != nil {
				utils.LogError("获取项目信息案场失败: " + err.Error())
				return nil, errors.New("获取项目信息失败")
			}
			courses[inx].CaseInfo = caseinfo
		}

		info.Courses = courses

		imgs, err := s.dao.GetCmsCaseImgs(info.CmsCaseId)
		if err != nil {
			utils.LogError("获取项目信息失败: " + err.Error())
			return nil, errors.New("获取项目信息失败")
		}
		info.CmsCaseImgs = imgs
	}

	return info, nil
}

// SaveCmsCase 保存项目信息
func (s *MessageServ) SaveCmsCase(cmscase model.TaCmsCase, courseids, imgs, detailimgs string) (*model.TaCmsCase, error) {
	var newInfo *model.TaCmsCase
	var err error
	if cmscase.Name == "" {
		return nil, utils.LogError("项目名称不能为空！")
	}
	if cmscase.CaseId == "" {
		return nil, utils.LogError("案场不能为空")
	}
	if imgs == "" {
		return nil, utils.LogError("图片不能为空")
	}
	if detailimgs == "" {
		return nil, utils.LogError("详情图片不能为空")
	}
	if cmscase.DetailContent == "" {
		return nil, utils.LogError("专题介绍不能为空")
	}

	// if cmscase.Title == "" {
	// 	return nil, utils.LogError("项目标题不能为空！")
	// }

	if cmscase.CmsCaseId == "" {
		org := s.ctx.Get("org").(model.SysOrg)
		cmscase.OrgId = org.OrgId
		newInfo, err = s.dao.AddCmsCase(cmscase)
	} else {
		err = s.dao.UpdateCmsCase(cmscase)
		newInfo = &cmscase
	}
	if err != nil {
		utils.LogError("保存项目信息失败: " + err.Error())
		return nil, errors.New("保存项目信息失败")
	}
	err = s.dao.DelCmsCaseCourse(newInfo.CmsCaseId)
	if err != nil {
		utils.LogError("保存项目信息失败: " + err.Error())
		return nil, errors.New("保存项目信息失败")
	}
	if newInfo.IsAllCourse == 0 {
		err = s.dao.SaveCmsCaseCourse(newInfo.CmsCaseId, courseids)
		if err != nil {
			utils.LogError("保存项目信息失败: " + err.Error())
			return nil, errors.New("保存项目信息失败")
		}
	}
	err = s.dao.DelCmsCaseImg(newInfo.CmsCaseId)
	if err != nil {
		utils.LogError("保存项目信息失败: " + err.Error())
		return nil, errors.New("保存项目信息失败")
	}

	if imgs != "" {
		arrimg := strings.Split(imgs, ",")
		for index, img := range arrimg {
			CmsImg := model.TaCmsCaseImage{
				ImageType:    models.IMAGETYPE_COVER,
				Sort:         (index + 1),
				CaseImageUrl: img,
				CmsCaseId:    newInfo.CmsCaseId,
			}
			err = s.dao.SaveCmsCaseImg(CmsImg)
			if err != nil {
				utils.LogError("保存项目信息失败: " + err.Error())
				return nil, errors.New("保存项目信息失败")
			}
		}
	}
	if detailimgs != "" {
		arrdetailimg := strings.Split(detailimgs, ",")
		for index, img := range arrdetailimg {
			CmsImg := model.TaCmsCaseImage{
				ImageType:    models.IMAGETYPE_DETAIL,
				Sort:         (index + 1),
				CaseImageUrl: img,
				CmsCaseId:    newInfo.CmsCaseId,
			}
			err = s.dao.SaveCmsCaseImg(CmsImg)
			if err != nil {
				utils.LogError("保存项目信息失败: " + err.Error())
				return nil, errors.New("保存项目信息失败")
			}
		}
	}
	return newInfo, err
}

// DelCmsCase 刪除项目信息
func (s *MessageServ) DelCmsCase(cmscaseid string) error {
	err := s.dao.DelCmsCase(cmscaseid)
	if err != nil {
		utils.LogError("删除项目信息失败: " + err.Error())
		return errors.New("删除项目信息失败")
	}
	return nil
}

// CmsCaseShow 前台展示
func (s *MessageServ) CmsCaseShow(cmscaseid string) error {
	err := s.dao.CmsCaseShow(cmscaseid)
	if err != nil {
		utils.LogError("修改项目信息失败: " + err.Error())
		return errors.New("修改项目信息失败")
	}
	return err
}

// CmsCaseHide 前台隐藏
func (s *MessageServ) CmsCaseHide(cmscaseid string) error {
	err := s.dao.CmsCaseHide(cmscaseid)
	if err != nil {
		utils.LogError("修改项目信息失败: " + err.Error())
		return errors.New("修改项目信息失败")
	}
	return err
}
