package message

import (
	"errors"
	"spaceofcheng/services/models/message"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strings"
)

// GetLocationList 获取列表
func (s *MessageServ) GetLocationList(issys, orgid string) ([]model.TdCmsImageLocation, error) {
	list, err := s.dao.GetLocationList(issys, orgid)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	return list, nil
}

// GetLocationByID 获取详细信息
func (s *MessageServ) GetLocationByID(id string) (*model.TdCmsImageLocation, error) {
	location, err := s.dao.GetLocationById(id)
	if err != nil {
		utils.LogError("获取信息失败: " + err.Error())
		return nil, errors.New("获取信息失败")
	}
	return location, err
}

// SaveLocation 保存
func (s *MessageServ) SaveLocation(location model.TdCmsImageLocation) (*model.TdCmsImageLocation, error) {
	if location.LocationId == "" {
		sort, err := s.dao.GetLocationMaxSort("0", location.OrgId)
		if err != nil {
			utils.LogError("保存信息失败: " + err.Error())
			return nil, errors.New("保存信息失败")
		}
		location.OrderNo = sort + 1
		newlocation, err := s.dao.AddLocation(location)
		return newlocation, err
	}
	err := s.dao.UpdateLocation(location)
	if err != nil {
		utils.LogError("保存信息失败: " + err.Error())
		return nil, errors.New("保存信息失败")
	}
	return &location, err
}

// DelLocation 删除
func (s *MessageServ) DelLocation(locationID string) error {
	loc, err := s.dao.GetLocationById(locationID)
	if err != nil {
		utils.LogError("删除信息失败: " + err.Error())
		return errors.New("删除信息失败")
	}
	err = s.dao.UpdateLocationSortByID(loc.OrderNo)
	if err != nil {
		utils.LogError("删除信息失败: " + err.Error())
		return errors.New("删除信息失败")
	}
	err = s.dao.DelLocation(locationID)
	if err != nil {
		utils.LogError("删除信息失败: " + err.Error())
		return errors.New("删除信息失败")
	}
	return nil
}

// UpdateLocationSort 更新顺序
func (s *MessageServ) UpdateLocationSort(locations string) error {
	if locations == "" {
		return utils.LogError("没有需要修改的5A信息！")
	}
	arrlocation := strings.Split(locations, ",")
	for _, loc := range arrlocation {
		loc := strings.Split(loc, ":")
		if len(loc) < 2 {
			return utils.LogError("修改信息不全！")
		}
		err := s.dao.UpdateLocationSort(loc)
		if err != nil {
			utils.LogError("更新失败: " + err.Error())
			return errors.New("更新失败")
		}
	}
	return nil
}

// GetLocationInfoList 获取详情
func (s *MessageServ) GetLocationInfoList() ([]message.LocationInfo, error) {
	var org = s.ctx.Get("org").(model.SysOrg)
	locations, err := s.dao.GetLocationInfoList(org.OrgId)
	if err != nil {
		utils.LogError("获取Location失败: " + err.Error())
		return nil, errors.New("获取失败")
	}
	for inx, l := range locations {
		news, err := s.dao.GetNewsByLocationShow(l.LocationId, org.OrgId)
		if err != nil {
			utils.LogError("获取新闻信息失败: " + err.Error())
			return nil, errors.New("获取新闻信息失败")
		}
		locations[inx].News = news

		imgs, err := s.dao.GetImgByLocation(l.LocationId, org.OrgId)
		if err != nil {
			utils.LogError("获取图片信息失败: " + err.Error())
			return nil, errors.New("获取图片信息失败")
		}
		locations[inx].Banners = imgs
		if l.LocationId == "selected" {
			courses, err := s.coursedao.GetSelectCourseWithCaseList(org.OrgId)
			if err != nil {
				utils.LogError("获取课程信息失败: " + err.Error())
				return nil, errors.New("获取课程信息失败")
			}
			locations[inx].Courses = courses
		} else {
			courses, err := s.coursedao.GetCourseWithCaseByLocation(org.OrgId, l.LocationId)
			if err != nil {
				utils.LogError("获取课程信息失败: " + err.Error())
				return nil, errors.New("获取课程信息失败")
			}
			locations[inx].Courses = courses
		}

		for ci, cc := range locations[inx].Courses {
			tags, err := s.coursedao.GetCourseTag(cc.CourseId)
			if err != nil {
				utils.LogError("获取课程标签信息失败: " + err.Error())
				return nil, errors.New("获取课程标签信息失败")
			}
			locations[inx].Courses[ci].CourseTags = tags
		}
	}

	return locations, nil
}
