package message

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/message"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"time"
)

// GetNewsList 获取咨询列表
func (s *MessageServ) GetNewsList(locationid, title, orgid string, page int, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	infos, err := s.dao.GetNewsList(locationid, title, orgid, page, pageSize)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	total, err := s.dao.GetNewsListCount(locationid, title, orgid)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}

	return map[string]interface{}{
		"list":     infos,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// GetNewsByLocation 根据位置id获取咨询信息
func (s *MessageServ) GetNewsByLocation(locationid, orgid string) ([]model.TaCmsNews, error) {
	news, err := s.dao.GetNewsByLocationShow(locationid, orgid)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	return news, err
}

// GetNewsByID 获取咨询详情
func (s *MessageServ) GetNewsByID(newsid string) (*message.CmsNews, error) {
	info, err := s.dao.GetNewsByID(newsid)
	if err != nil {
		utils.LogError("获取咨询信息失败: " + err.Error())
		return nil, errors.New("获取咨询信息失败")
	}
	newslocation, err := s.dao.GetNewsLocationByID(newsid)
	if err != nil {
		utils.LogError("获取咨询信息失败: " + err.Error())
		return nil, errors.New("获取咨询信息失败")
	}
	info.NewsLocations = newslocation
	return info, nil
}

// SaveNews 保存咨询信息
func (s *MessageServ) SaveNews(news model.TaCmsNews, locationids string) (*model.TaCmsNews, error) {
	var newInfo *model.TaCmsNews
	var err error
	if news.Title == "" {
		return nil, errors.New("咨询标题不允许为空！")
	}

	if news.ImageUrl == "" {
		return nil, errors.New("图片不允许为空！")
	}

	if news.ForwardType == "" {
		return nil, errors.New("跳转类型不允许为空！")
	}
	if news.ForwardType == ForwardTypeUrl && news.ForwardUrl == "" {
		return nil, errors.New("跳转地址不允许为空！")
	}

	if news.ForwardType == ForwardTypeCourse && news.ForwardCourseId == "" {
		return nil, errors.New("课程不允许为空！")
	}

	if news.Status == models.STATUS_NORMAL {
		miniDT := time.Date(2000, time.January, 1, 0, 0, 0, 0, time.Local)
		if news.PublicDate.Before(miniDT) {
			news.PublicDate = time.Now()
		}
	}

	if news.NewsId == "" {
		newInfo, err = s.dao.AddNews(news)
	} else {
		err = s.dao.UpdateNews(news)
		newInfo = &news
	}
	if err != nil {
		utils.LogError("保存咨询信息失败: " + err.Error())
		return nil, errors.New("保存咨询信息失败")
	}
	err = s.dao.DelNewsLocation(newInfo.NewsId)
	if err != nil {
		utils.LogError("保存咨询信息失败: " + err.Error())
		return nil, errors.New("保存咨询信息失败")
	}
	err = s.dao.SaveNewsLocation(newInfo.NewsId, locationids)
	if err != nil {
		utils.LogError("保存咨询信息失败: " + err.Error())
		return nil, errors.New("保存咨询信息失败")
	}
	return newInfo, err
}

// DelNews 刪除咨询信息
func (s *MessageServ) DelNews(newsid string) error {
	err := s.dao.DelNews(newsid)
	if err != nil {
		utils.LogError("删除咨询信息失败: " + err.Error())
		return errors.New("删除咨询信息失败")
	}
	return err
}
