package statistics

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/statistics"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// GetGoodsStatistics 获取商品统计信息
func (s *StatisticsServ) GetGoodsStatistics(caseids, typeid, name string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}

	if caseids == "" {
		return nil, errors.New("请先选择案场信息")
	}

	list, err := s.dao.GetGoodsStatistics(caseids, typeid, name, page, pageSize)
	if err != nil {
		utils.LogError("获取商品统计数据失败: " + err.Error())
		return nil, errors.New("获取商品统计数据失败")
	}
	all, err := s.dao.GetGoodsStatisticsCount(caseids, typeid, name)
	if err != nil {
		utils.LogError("获取商品统计数据失败: " + err.Error())
		return nil, errors.New("获取商品统计数据失败")
	}

	return map[string]interface{}{
		"list":     list,
		"pagesize": pageSize,
		"pagenum":  len(all),
		"page":     page,
	}, nil
}

// GetGoodsStatisticsExcel 获取商品统计信息导出
func (s *StatisticsServ) GetGoodsStatisticsExcel(caseids, typeid, name string) ([]statistics.GoodsStatistics, error) {
	all, err := s.dao.GetGoodsStatisticsCount(caseids, typeid, name)
	if err != nil {
		utils.LogError("获取商品统计数据失败: " + err.Error())
		return nil, errors.New("获取商品统计数据失败")
	}
	return all, nil
}

// GetGoodsOrdersStatistics 获取商品订单统计信息
func (s *StatisticsServ) GetGoodsOrdersStatistics(status, usertype, caseids, begindate, enddate, paytype string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}

	if caseids == "" {
		return nil, errors.New("请先选择案场信息")
	}

	list, total, err := s.dao.GetGoodsOrderStatistics(status, usertype, caseids, begindate, enddate, paytype, page, pageSize)
	if err != nil {
		utils.LogError("获取商品订单统计数据失败: " + err.Error())
		return nil, errors.New("获取商品订单统计数据失败")
	}
	for a := 0; a < len(list); a++ {
		switch list[a].PayType {
		case models.CONSUME_INNER:
			list[a].PayTypeStr = "案场"
		case models.CONSUME_COUPON:
			list[a].PayTypeStr = "优惠券"
		case models.CONSUME_COINCHG:
			list[a].PayTypeStr = "城币"
		case models.CONSUME_POINTS:
			list[a].PayTypeStr = "积分"
		}
	}

	return map[string]interface{}{
		"list":     list,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, nil
}

// GetGoodsOrdersStatisticsExcel 商品订单导出
func (s *StatisticsServ) GetGoodsOrdersStatisticsExcel(status, usertype, caseids, begindate, enddate, paytype string) ([]statistics.GoodsOrdersStatistics, error) {
	if caseids == "" {
		return nil, errors.New("请先选择案场信息")
	}

	all, err := s.dao.GetGoodsOrderStatisticsCount(status, usertype, caseids, begindate, enddate, paytype)
	if err != nil {
		utils.LogError("获取商品订单统计数据失败: " + err.Error())
		return nil, errors.New("获取商品订单统计数据失败")
	}
	for inx, item := range all {
		details, err := s.dao.GetGoodsOrderDetailByOrderId(item.OrdersId)
		if err != nil {
			return nil, err
		}
		all[inx].Details = details
	}
	return all, nil
}
