package utils

import (
	"math/rand"
	"reflect"
	"strconv"
	"strings"
	"time"
)

var rnd = rand.New(rand.NewSource(time.Now().UnixNano()))

// Shuffle 随机数组
func Shuffle(arr interface{}) interface{} {
	newSeed()

	v := reflect.ValueOf(arr)

	arrLen := v.Len()
	res := reflect.MakeSlice(v.Type(), arrLen, arrLen)
	reflect.Copy(res, v)

	for i := 0; i < arrLen; i++ {
		m := rnd.Intn(arrLen)
		n := arrLen - m - 1

		swp := res.Index(m)
		res.Index(m).Set(res.Index(n))
		res.Index(n).Set(swp)
	}

	return res.Interface()
}

// GetRand 获取随机数
func GetRand(len int) string {
	newSeed()
	var r []string
	for i := 0; i < len; i++ {
		m := rnd.Intn(9)
		r = append(r, strconv.Itoa(m))
	}
	return strings.Join(r, "")
}

// LuckyDraw 抽奖
func LuckyDraw(pList []map[string]interface{}) interface{} {
	// 奖池
	pool := make([]interface{}, 0)

	// 生成奖池
	for _, it := range pList {
		prize := it["prize"]
		prob := it["prob"].(int)

		for i := 0; i < prob; i++ {
			pool = append(pool, prize)
		}
	}

	poolLen := len(pool)
	if poolLen == 0 {
		return nil
	}

	// 打乱奖池 - 打乱两次, 以使其更混乱, 更随机
	pool = Shuffle(pool).([]interface{})

	newSeed()
	inx := rnd.Intn(poolLen)
	return pool[inx]
}

func newSeed() {
	seed := time.Now().UnixNano() + int64(rnd.Intn(1000)*1000)
	rnd.Seed(seed)
}
